"""Game7 Orbit, Arbitrum, Xai and Mantle blockchain indexes models

Revision ID: a4ef4f9031e4
Revises: 0ea24314c181
Create Date: 2024-05-29 13:19:42.833716

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = 'a4ef4f9031e4'
down_revision: Union[str, None] = '0ea24314c181'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('arbitrum_one_blocks',
    sa.Column('l1_block_number', sa.BigInteger(), nullable=False),
    sa.Column('block_number', sa.BigInteger(), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('block_timestamp', sa.BigInteger(), nullable=False),
    sa.Column('parent_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('row_id', sa.BigInteger(), nullable=False),
    sa.Column('path', sa.Text(), nullable=False),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.PrimaryKeyConstraint('block_number', name=op.f('pk_arbitrum_one_blocks'))
    )
    op.create_index(op.f('ix_arbitrum_one_blocks_block_number'), 'arbitrum_one_blocks', ['block_number'], unique=False)
    op.create_index(op.f('ix_arbitrum_one_blocks_block_timestamp'), 'arbitrum_one_blocks', ['block_timestamp'], unique=False)
    op.create_table('arbitrum_one_reorgs',
    sa.Column('id', sa.UUID(), nullable=False),
    sa.Column('block_number', sa.BigInteger(), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.PrimaryKeyConstraint('id', name=op.f('pk_arbitrum_one_reorgs'))
    )
    op.create_index(op.f('ix_arbitrum_one_reorgs_block_hash'), 'arbitrum_one_reorgs', ['block_hash'], unique=False)
    op.create_index(op.f('ix_arbitrum_one_reorgs_block_number'), 'arbitrum_one_reorgs', ['block_number'], unique=False)
    op.create_table('arbitrum_sepolia_blocks',
    sa.Column('l1_block_number', sa.BigInteger(), nullable=False),
    sa.Column('block_number', sa.BigInteger(), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('block_timestamp', sa.BigInteger(), nullable=False),
    sa.Column('parent_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('row_id', sa.BigInteger(), nullable=False),
    sa.Column('path', sa.Text(), nullable=False),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.PrimaryKeyConstraint('block_number', name=op.f('pk_arbitrum_sepolia_blocks'))
    )
    op.create_index(op.f('ix_arbitrum_sepolia_blocks_block_number'), 'arbitrum_sepolia_blocks', ['block_number'], unique=False)
    op.create_index(op.f('ix_arbitrum_sepolia_blocks_block_timestamp'), 'arbitrum_sepolia_blocks', ['block_timestamp'], unique=False)
    op.create_table('arbitrum_sepolia_reorgs',
    sa.Column('id', sa.UUID(), nullable=False),
    sa.Column('block_number', sa.BigInteger(), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.PrimaryKeyConstraint('id', name=op.f('pk_arbitrum_sepolia_reorgs'))
    )
    op.create_index(op.f('ix_arbitrum_sepolia_reorgs_block_hash'), 'arbitrum_sepolia_reorgs', ['block_hash'], unique=False)
    op.create_index(op.f('ix_arbitrum_sepolia_reorgs_block_number'), 'arbitrum_sepolia_reorgs', ['block_number'], unique=False)
    op.create_table('game7_orbit_arbitrum_sepolia_blocks',
    sa.Column('l1_block_number', sa.BigInteger(), nullable=False),
    sa.Column('block_number', sa.BigInteger(), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('block_timestamp', sa.BigInteger(), nullable=False),
    sa.Column('parent_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('row_id', sa.BigInteger(), nullable=False),
    sa.Column('path', sa.Text(), nullable=False),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.PrimaryKeyConstraint('block_number', name=op.f('pk_game7_orbit_arbitrum_sepolia_blocks'))
    )
    op.create_index(op.f('ix_game7_orbit_arbitrum_sepolia_blocks_block_number'), 'game7_orbit_arbitrum_sepolia_blocks', ['block_number'], unique=False)
    op.create_index(op.f('ix_game7_orbit_arbitrum_sepolia_blocks_block_timestamp'), 'game7_orbit_arbitrum_sepolia_blocks', ['block_timestamp'], unique=False)
    op.create_table('game7_orbit_arbitrum_sepolia_reorgs',
    sa.Column('id', sa.UUID(), nullable=False),
    sa.Column('block_number', sa.BigInteger(), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.PrimaryKeyConstraint('id', name=op.f('pk_game7_orbit_arbitrum_sepolia_reorgs'))
    )
    op.create_index(op.f('ix_game7_orbit_arbitrum_sepolia_reorgs_block_hash'), 'game7_orbit_arbitrum_sepolia_reorgs', ['block_hash'], unique=False)
    op.create_index(op.f('ix_game7_orbit_arbitrum_sepolia_reorgs_block_number'), 'game7_orbit_arbitrum_sepolia_reorgs', ['block_number'], unique=False)
    op.create_table('mantle_blocks',
    sa.Column('block_number', sa.BigInteger(), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('block_timestamp', sa.BigInteger(), nullable=False),
    sa.Column('parent_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('row_id', sa.BigInteger(), nullable=False),
    sa.Column('path', sa.Text(), nullable=False),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.PrimaryKeyConstraint('block_number', name=op.f('pk_mantle_blocks'))
    )
    op.create_index(op.f('ix_mantle_blocks_block_number'), 'mantle_blocks', ['block_number'], unique=False)
    op.create_index(op.f('ix_mantle_blocks_block_timestamp'), 'mantle_blocks', ['block_timestamp'], unique=False)
    op.create_table('mantle_reorgs',
    sa.Column('id', sa.UUID(), nullable=False),
    sa.Column('block_number', sa.BigInteger(), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.PrimaryKeyConstraint('id', name=op.f('pk_mantle_reorgs'))
    )
    op.create_index(op.f('ix_mantle_reorgs_block_hash'), 'mantle_reorgs', ['block_hash'], unique=False)
    op.create_index(op.f('ix_mantle_reorgs_block_number'), 'mantle_reorgs', ['block_number'], unique=False)
    op.create_table('mantle_sepolia_blocks',
    sa.Column('block_number', sa.BigInteger(), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('block_timestamp', sa.BigInteger(), nullable=False),
    sa.Column('parent_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('row_id', sa.BigInteger(), nullable=False),
    sa.Column('path', sa.Text(), nullable=False),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.PrimaryKeyConstraint('block_number', name=op.f('pk_mantle_sepolia_blocks'))
    )
    op.create_index(op.f('ix_mantle_sepolia_blocks_block_number'), 'mantle_sepolia_blocks', ['block_number'], unique=False)
    op.create_index(op.f('ix_mantle_sepolia_blocks_block_timestamp'), 'mantle_sepolia_blocks', ['block_timestamp'], unique=False)
    op.create_table('mantle_sepolia_reorgs',
    sa.Column('id', sa.UUID(), nullable=False),
    sa.Column('block_number', sa.BigInteger(), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.PrimaryKeyConstraint('id', name=op.f('pk_mantle_sepolia_reorgs'))
    )
    op.create_index(op.f('ix_mantle_sepolia_reorgs_block_hash'), 'mantle_sepolia_reorgs', ['block_hash'], unique=False)
    op.create_index(op.f('ix_mantle_sepolia_reorgs_block_number'), 'mantle_sepolia_reorgs', ['block_number'], unique=False)
    op.create_table('xai_blocks',
    sa.Column('l1_block_number', sa.BigInteger(), nullable=False),
    sa.Column('block_number', sa.BigInteger(), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('block_timestamp', sa.BigInteger(), nullable=False),
    sa.Column('parent_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('row_id', sa.BigInteger(), nullable=False),
    sa.Column('path', sa.Text(), nullable=False),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.PrimaryKeyConstraint('block_number', name=op.f('pk_xai_blocks'))
    )
    op.create_index(op.f('ix_xai_blocks_block_number'), 'xai_blocks', ['block_number'], unique=False)
    op.create_index(op.f('ix_xai_blocks_block_timestamp'), 'xai_blocks', ['block_timestamp'], unique=False)
    op.create_table('xai_reorgs',
    sa.Column('id', sa.UUID(), nullable=False),
    sa.Column('block_number', sa.BigInteger(), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.PrimaryKeyConstraint('id', name=op.f('pk_xai_reorgs'))
    )
    op.create_index(op.f('ix_xai_reorgs_block_hash'), 'xai_reorgs', ['block_hash'], unique=False)
    op.create_index(op.f('ix_xai_reorgs_block_number'), 'xai_reorgs', ['block_number'], unique=False)
    op.create_table('xai_sepolia_blocks',
    sa.Column('l1_block_number', sa.BigInteger(), nullable=False),
    sa.Column('block_number', sa.BigInteger(), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('block_timestamp', sa.BigInteger(), nullable=False),
    sa.Column('parent_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('row_id', sa.BigInteger(), nullable=False),
    sa.Column('path', sa.Text(), nullable=False),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.PrimaryKeyConstraint('block_number', name=op.f('pk_xai_sepolia_blocks'))
    )
    op.create_index(op.f('ix_xai_sepolia_blocks_block_number'), 'xai_sepolia_blocks', ['block_number'], unique=False)
    op.create_index(op.f('ix_xai_sepolia_blocks_block_timestamp'), 'xai_sepolia_blocks', ['block_timestamp'], unique=False)
    op.create_table('xai_sepolia_reorgs',
    sa.Column('id', sa.UUID(), nullable=False),
    sa.Column('block_number', sa.BigInteger(), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.PrimaryKeyConstraint('id', name=op.f('pk_xai_sepolia_reorgs'))
    )
    op.create_index(op.f('ix_xai_sepolia_reorgs_block_hash'), 'xai_sepolia_reorgs', ['block_hash'], unique=False)
    op.create_index(op.f('ix_xai_sepolia_reorgs_block_number'), 'xai_sepolia_reorgs', ['block_number'], unique=False)
    op.create_table('arbitrum_one_transactions',
    sa.Column('block_number', sa.BigInteger(), nullable=False),
    sa.Column('hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('from_address', sa.LargeBinary(length=20), nullable=False),
    sa.Column('to_address', sa.LargeBinary(length=20), nullable=False),
    sa.Column('selector', sa.VARCHAR(length=256), nullable=True),
    sa.Column('type', sa.Integer(), nullable=True),
    sa.Column('row_id', sa.BigInteger(), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('index', sa.BigInteger(), nullable=False),
    sa.Column('path', sa.Text(), nullable=False),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.ForeignKeyConstraint(['block_number'], ['arbitrum_one_blocks.block_number'], name=op.f('fk_arbitrum_one_transactions_block_number_arbitrum_one_blocks'), ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('hash', name=op.f('pk_arbitrum_one_transactions'))
    )
    op.create_index(op.f('ix_arbitrum_one_transactions_block_hash'), 'arbitrum_one_transactions', ['block_hash'], unique=False)
    op.create_index(op.f('ix_arbitrum_one_transactions_block_number'), 'arbitrum_one_transactions', ['block_number'], unique=False)
    op.create_index(op.f('ix_arbitrum_one_transactions_from_address'), 'arbitrum_one_transactions', ['from_address'], unique=False)
    op.create_index(op.f('ix_arbitrum_one_transactions_hash'), 'arbitrum_one_transactions', ['hash'], unique=True)
    op.create_index(op.f('ix_arbitrum_one_transactions_index'), 'arbitrum_one_transactions', ['index'], unique=False)
    op.create_index(op.f('ix_arbitrum_one_transactions_selector'), 'arbitrum_one_transactions', ['selector'], unique=False)
    op.create_index(op.f('ix_arbitrum_one_transactions_to_address'), 'arbitrum_one_transactions', ['to_address'], unique=False)
    op.create_index(op.f('ix_arbitrum_one_transactions_type'), 'arbitrum_one_transactions', ['type'], unique=False)
    op.create_table('arbitrum_sepolia_transactions',
    sa.Column('block_number', sa.BigInteger(), nullable=False),
    sa.Column('hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('from_address', sa.LargeBinary(length=20), nullable=False),
    sa.Column('to_address', sa.LargeBinary(length=20), nullable=False),
    sa.Column('selector', sa.VARCHAR(length=256), nullable=True),
    sa.Column('type', sa.Integer(), nullable=True),
    sa.Column('row_id', sa.BigInteger(), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('index', sa.BigInteger(), nullable=False),
    sa.Column('path', sa.Text(), nullable=False),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.ForeignKeyConstraint(['block_number'], ['arbitrum_sepolia_blocks.block_number'], name=op.f('fk_arbitrum_sepolia_transactions_block_number_arbitrum_sepolia_blocks'), ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('hash', name=op.f('pk_arbitrum_sepolia_transactions'))
    )
    op.create_index(op.f('ix_arbitrum_sepolia_transactions_block_hash'), 'arbitrum_sepolia_transactions', ['block_hash'], unique=False)
    op.create_index(op.f('ix_arbitrum_sepolia_transactions_block_number'), 'arbitrum_sepolia_transactions', ['block_number'], unique=False)
    op.create_index(op.f('ix_arbitrum_sepolia_transactions_from_address'), 'arbitrum_sepolia_transactions', ['from_address'], unique=False)
    op.create_index(op.f('ix_arbitrum_sepolia_transactions_hash'), 'arbitrum_sepolia_transactions', ['hash'], unique=True)
    op.create_index(op.f('ix_arbitrum_sepolia_transactions_index'), 'arbitrum_sepolia_transactions', ['index'], unique=False)
    op.create_index(op.f('ix_arbitrum_sepolia_transactions_selector'), 'arbitrum_sepolia_transactions', ['selector'], unique=False)
    op.create_index(op.f('ix_arbitrum_sepolia_transactions_to_address'), 'arbitrum_sepolia_transactions', ['to_address'], unique=False)
    op.create_index(op.f('ix_arbitrum_sepolia_transactions_type'), 'arbitrum_sepolia_transactions', ['type'], unique=False)
    op.create_table('game7_orbit_arbitrum_sepolia_transactions',
    sa.Column('block_number', sa.BigInteger(), nullable=False),
    sa.Column('hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('from_address', sa.LargeBinary(length=20), nullable=False),
    sa.Column('to_address', sa.LargeBinary(length=20), nullable=False),
    sa.Column('selector', sa.VARCHAR(length=256), nullable=True),
    sa.Column('type', sa.Integer(), nullable=True),
    sa.Column('row_id', sa.BigInteger(), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('index', sa.BigInteger(), nullable=False),
    sa.Column('path', sa.Text(), nullable=False),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.ForeignKeyConstraint(['block_number'], ['game7_orbit_arbitrum_sepolia_blocks.block_number'], name=op.f('fk_game7_orbit_arbitrum_sepolia_transactions_block_number_game7_orbit_arbitrum_sepolia_blocks'), ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('hash', name=op.f('pk_game7_orbit_arbitrum_sepolia_transactions'))
    )
    op.create_index(op.f('ix_game7_orbit_arbitrum_sepolia_transactions_block_hash'), 'game7_orbit_arbitrum_sepolia_transactions', ['block_hash'], unique=False)
    op.create_index(op.f('ix_game7_orbit_arbitrum_sepolia_transactions_block_number'), 'game7_orbit_arbitrum_sepolia_transactions', ['block_number'], unique=False)
    op.create_index(op.f('ix_game7_orbit_arbitrum_sepolia_transactions_from_address'), 'game7_orbit_arbitrum_sepolia_transactions', ['from_address'], unique=False)
    op.create_index(op.f('ix_game7_orbit_arbitrum_sepolia_transactions_hash'), 'game7_orbit_arbitrum_sepolia_transactions', ['hash'], unique=True)
    op.create_index(op.f('ix_game7_orbit_arbitrum_sepolia_transactions_index'), 'game7_orbit_arbitrum_sepolia_transactions', ['index'], unique=False)
    op.create_index(op.f('ix_game7_orbit_arbitrum_sepolia_transactions_selector'), 'game7_orbit_arbitrum_sepolia_transactions', ['selector'], unique=False)
    op.create_index(op.f('ix_game7_orbit_arbitrum_sepolia_transactions_to_address'), 'game7_orbit_arbitrum_sepolia_transactions', ['to_address'], unique=False)
    op.create_index(op.f('ix_game7_orbit_arbitrum_sepolia_transactions_type'), 'game7_orbit_arbitrum_sepolia_transactions', ['type'], unique=False)
    op.create_table('mantle_sepolia_transactions',
    sa.Column('block_number', sa.BigInteger(), nullable=False),
    sa.Column('hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('from_address', sa.LargeBinary(length=20), nullable=False),
    sa.Column('to_address', sa.LargeBinary(length=20), nullable=False),
    sa.Column('selector', sa.VARCHAR(length=256), nullable=True),
    sa.Column('type', sa.Integer(), nullable=True),
    sa.Column('row_id', sa.BigInteger(), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('index', sa.BigInteger(), nullable=False),
    sa.Column('path', sa.Text(), nullable=False),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.ForeignKeyConstraint(['block_number'], ['mantle_sepolia_blocks.block_number'], name=op.f('fk_mantle_sepolia_transactions_block_number_mantle_sepolia_blocks'), ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('hash', name=op.f('pk_mantle_sepolia_transactions'))
    )
    op.create_index(op.f('ix_mantle_sepolia_transactions_block_hash'), 'mantle_sepolia_transactions', ['block_hash'], unique=False)
    op.create_index(op.f('ix_mantle_sepolia_transactions_block_number'), 'mantle_sepolia_transactions', ['block_number'], unique=False)
    op.create_index(op.f('ix_mantle_sepolia_transactions_from_address'), 'mantle_sepolia_transactions', ['from_address'], unique=False)
    op.create_index(op.f('ix_mantle_sepolia_transactions_hash'), 'mantle_sepolia_transactions', ['hash'], unique=True)
    op.create_index(op.f('ix_mantle_sepolia_transactions_index'), 'mantle_sepolia_transactions', ['index'], unique=False)
    op.create_index(op.f('ix_mantle_sepolia_transactions_selector'), 'mantle_sepolia_transactions', ['selector'], unique=False)
    op.create_index(op.f('ix_mantle_sepolia_transactions_to_address'), 'mantle_sepolia_transactions', ['to_address'], unique=False)
    op.create_index(op.f('ix_mantle_sepolia_transactions_type'), 'mantle_sepolia_transactions', ['type'], unique=False)
    op.create_table('mantle_transactions',
    sa.Column('block_number', sa.BigInteger(), nullable=False),
    sa.Column('hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('from_address', sa.LargeBinary(length=20), nullable=False),
    sa.Column('to_address', sa.LargeBinary(length=20), nullable=False),
    sa.Column('selector', sa.VARCHAR(length=256), nullable=True),
    sa.Column('type', sa.Integer(), nullable=True),
    sa.Column('row_id', sa.BigInteger(), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('index', sa.BigInteger(), nullable=False),
    sa.Column('path', sa.Text(), nullable=False),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.ForeignKeyConstraint(['block_number'], ['mantle_blocks.block_number'], name=op.f('fk_mantle_transactions_block_number_mantle_blocks'), ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('hash', name=op.f('pk_mantle_transactions'))
    )
    op.create_index(op.f('ix_mantle_transactions_block_hash'), 'mantle_transactions', ['block_hash'], unique=False)
    op.create_index(op.f('ix_mantle_transactions_block_number'), 'mantle_transactions', ['block_number'], unique=False)
    op.create_index(op.f('ix_mantle_transactions_from_address'), 'mantle_transactions', ['from_address'], unique=False)
    op.create_index(op.f('ix_mantle_transactions_hash'), 'mantle_transactions', ['hash'], unique=True)
    op.create_index(op.f('ix_mantle_transactions_index'), 'mantle_transactions', ['index'], unique=False)
    op.create_index(op.f('ix_mantle_transactions_selector'), 'mantle_transactions', ['selector'], unique=False)
    op.create_index(op.f('ix_mantle_transactions_to_address'), 'mantle_transactions', ['to_address'], unique=False)
    op.create_index(op.f('ix_mantle_transactions_type'), 'mantle_transactions', ['type'], unique=False)
    op.create_table('xai_sepolia_transactions',
    sa.Column('block_number', sa.BigInteger(), nullable=False),
    sa.Column('hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('from_address', sa.LargeBinary(length=20), nullable=False),
    sa.Column('to_address', sa.LargeBinary(length=20), nullable=False),
    sa.Column('selector', sa.VARCHAR(length=256), nullable=True),
    sa.Column('type', sa.Integer(), nullable=True),
    sa.Column('row_id', sa.BigInteger(), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('index', sa.BigInteger(), nullable=False),
    sa.Column('path', sa.Text(), nullable=False),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.ForeignKeyConstraint(['block_number'], ['xai_sepolia_blocks.block_number'], name=op.f('fk_xai_sepolia_transactions_block_number_xai_sepolia_blocks'), ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('hash', name=op.f('pk_xai_sepolia_transactions'))
    )
    op.create_index(op.f('ix_xai_sepolia_transactions_block_hash'), 'xai_sepolia_transactions', ['block_hash'], unique=False)
    op.create_index(op.f('ix_xai_sepolia_transactions_block_number'), 'xai_sepolia_transactions', ['block_number'], unique=False)
    op.create_index(op.f('ix_xai_sepolia_transactions_from_address'), 'xai_sepolia_transactions', ['from_address'], unique=False)
    op.create_index(op.f('ix_xai_sepolia_transactions_hash'), 'xai_sepolia_transactions', ['hash'], unique=True)
    op.create_index(op.f('ix_xai_sepolia_transactions_index'), 'xai_sepolia_transactions', ['index'], unique=False)
    op.create_index(op.f('ix_xai_sepolia_transactions_selector'), 'xai_sepolia_transactions', ['selector'], unique=False)
    op.create_index(op.f('ix_xai_sepolia_transactions_to_address'), 'xai_sepolia_transactions', ['to_address'], unique=False)
    op.create_index(op.f('ix_xai_sepolia_transactions_type'), 'xai_sepolia_transactions', ['type'], unique=False)
    op.create_table('xai_transactions',
    sa.Column('block_number', sa.BigInteger(), nullable=False),
    sa.Column('hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('from_address', sa.LargeBinary(length=20), nullable=False),
    sa.Column('to_address', sa.LargeBinary(length=20), nullable=False),
    sa.Column('selector', sa.VARCHAR(length=256), nullable=True),
    sa.Column('type', sa.Integer(), nullable=True),
    sa.Column('row_id', sa.BigInteger(), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('index', sa.BigInteger(), nullable=False),
    sa.Column('path', sa.Text(), nullable=False),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.ForeignKeyConstraint(['block_number'], ['xai_blocks.block_number'], name=op.f('fk_xai_transactions_block_number_xai_blocks'), ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('hash', name=op.f('pk_xai_transactions'))
    )
    op.create_index(op.f('ix_xai_transactions_block_hash'), 'xai_transactions', ['block_hash'], unique=False)
    op.create_index(op.f('ix_xai_transactions_block_number'), 'xai_transactions', ['block_number'], unique=False)
    op.create_index(op.f('ix_xai_transactions_from_address'), 'xai_transactions', ['from_address'], unique=False)
    op.create_index(op.f('ix_xai_transactions_hash'), 'xai_transactions', ['hash'], unique=True)
    op.create_index(op.f('ix_xai_transactions_index'), 'xai_transactions', ['index'], unique=False)
    op.create_index(op.f('ix_xai_transactions_selector'), 'xai_transactions', ['selector'], unique=False)
    op.create_index(op.f('ix_xai_transactions_to_address'), 'xai_transactions', ['to_address'], unique=False)
    op.create_index(op.f('ix_xai_transactions_type'), 'xai_transactions', ['type'], unique=False)
    op.create_table('arbitrum_one_logs',
    sa.Column('transaction_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('address', sa.LargeBinary(length=20), nullable=False),
    sa.Column('row_id', sa.BigInteger(), nullable=False),
    sa.Column('selector', sa.VARCHAR(length=256), nullable=True),
    sa.Column('topic1', sa.VARCHAR(length=256), nullable=True),
    sa.Column('topic2', sa.VARCHAR(length=256), nullable=True),
    sa.Column('log_index', sa.BigInteger(), nullable=False),
    sa.Column('path', sa.Text(), nullable=False),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.ForeignKeyConstraint(['transaction_hash'], ['arbitrum_one_transactions.hash'], name=op.f('fk_arbitrum_one_logs_transaction_hash_arbitrum_one_transactions'), ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('transaction_hash', 'log_index', name='pk_arbitrum_one_log_index'),
    sa.UniqueConstraint('transaction_hash', 'log_index', name='uq_arbitrum_one_log_idx_tx_hash_log_idx')
    )
    op.create_index(op.f('ix_arbitrum_one_logs_address'), 'arbitrum_one_logs', ['address'], unique=False)
    op.create_index(op.f('ix_arbitrum_one_logs_block_hash'), 'arbitrum_one_logs', ['block_hash'], unique=False)
    op.create_index(op.f('ix_arbitrum_one_logs_transaction_hash'), 'arbitrum_one_logs', ['transaction_hash'], unique=False)
    op.create_table('arbitrum_sepolia_logs',
    sa.Column('transaction_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('address', sa.LargeBinary(length=20), nullable=False),
    sa.Column('row_id', sa.BigInteger(), nullable=False),
    sa.Column('selector', sa.VARCHAR(length=256), nullable=True),
    sa.Column('topic1', sa.VARCHAR(length=256), nullable=True),
    sa.Column('topic2', sa.VARCHAR(length=256), nullable=True),
    sa.Column('log_index', sa.BigInteger(), nullable=False),
    sa.Column('path', sa.Text(), nullable=False),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.ForeignKeyConstraint(['transaction_hash'], ['arbitrum_sepolia_transactions.hash'], name=op.f('fk_arbitrum_sepolia_logs_transaction_hash_arbitrum_sepolia_transactions'), ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('transaction_hash', 'log_index', name='pk_arbitrum_sepolia_log_index'),
    sa.UniqueConstraint('transaction_hash', 'log_index', name='uq_arbitrum_sepolia_log_idx_tx_hash_log_idx')
    )
    op.create_index(op.f('ix_arbitrum_sepolia_logs_address'), 'arbitrum_sepolia_logs', ['address'], unique=False)
    op.create_index(op.f('ix_arbitrum_sepolia_logs_block_hash'), 'arbitrum_sepolia_logs', ['block_hash'], unique=False)
    op.create_index(op.f('ix_arbitrum_sepolia_logs_transaction_hash'), 'arbitrum_sepolia_logs', ['transaction_hash'], unique=False)
    op.create_table('game7_orbit_arbitrum_sepolia_logs',
    sa.Column('transaction_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('address', sa.LargeBinary(length=20), nullable=False),
    sa.Column('row_id', sa.BigInteger(), nullable=False),
    sa.Column('selector', sa.VARCHAR(length=256), nullable=True),
    sa.Column('topic1', sa.VARCHAR(length=256), nullable=True),
    sa.Column('topic2', sa.VARCHAR(length=256), nullable=True),
    sa.Column('log_index', sa.BigInteger(), nullable=False),
    sa.Column('path', sa.Text(), nullable=False),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.ForeignKeyConstraint(['transaction_hash'], ['game7_orbit_arbitrum_sepolia_transactions.hash'], name=op.f('fk_game7_orbit_arbitrum_sepolia_logs_transaction_hash_game7_orbit_arbitrum_sepolia_transactions'), ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('transaction_hash', 'log_index', name='pk_game7_orbit_arbitrum_sepolia_log_index'),
    sa.UniqueConstraint('transaction_hash', 'log_index', name='uq_g7o_arbitrum_sepolia_log_idx_tx_hash_log_idx')
    )
    op.create_index(op.f('ix_game7_orbit_arbitrum_sepolia_logs_address'), 'game7_orbit_arbitrum_sepolia_logs', ['address'], unique=False)
    op.create_index(op.f('ix_game7_orbit_arbitrum_sepolia_logs_block_hash'), 'game7_orbit_arbitrum_sepolia_logs', ['block_hash'], unique=False)
    op.create_index(op.f('ix_game7_orbit_arbitrum_sepolia_logs_transaction_hash'), 'game7_orbit_arbitrum_sepolia_logs', ['transaction_hash'], unique=False)
    op.create_table('mantle_logs',
    sa.Column('transaction_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('address', sa.LargeBinary(length=20), nullable=False),
    sa.Column('row_id', sa.BigInteger(), nullable=False),
    sa.Column('selector', sa.VARCHAR(length=256), nullable=True),
    sa.Column('topic1', sa.VARCHAR(length=256), nullable=True),
    sa.Column('topic2', sa.VARCHAR(length=256), nullable=True),
    sa.Column('log_index', sa.BigInteger(), nullable=False),
    sa.Column('path', sa.Text(), nullable=False),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.ForeignKeyConstraint(['transaction_hash'], ['mantle_transactions.hash'], name=op.f('fk_mantle_logs_transaction_hash_mantle_transactions'), ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('transaction_hash', 'log_index', name='pk_mantle_log_index'),
    sa.UniqueConstraint('transaction_hash', 'log_index', name='uq_mantle_log_index_transaction_hash_log_index')
    )
    op.create_index('idx_mantle_logs_block_hash_log_index', 'mantle_logs', ['block_hash', 'log_index'], unique=True)
    op.create_index(op.f('ix_mantle_logs_address'), 'mantle_logs', ['address'], unique=False)
    op.create_index(op.f('ix_mantle_logs_block_hash'), 'mantle_logs', ['block_hash'], unique=False)
    op.create_index(op.f('ix_mantle_logs_transaction_hash'), 'mantle_logs', ['transaction_hash'], unique=False)
    op.create_table('mantle_sepolia_logs',
    sa.Column('transaction_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('address', sa.LargeBinary(length=20), nullable=False),
    sa.Column('row_id', sa.BigInteger(), nullable=False),
    sa.Column('selector', sa.VARCHAR(length=256), nullable=True),
    sa.Column('topic1', sa.VARCHAR(length=256), nullable=True),
    sa.Column('topic2', sa.VARCHAR(length=256), nullable=True),
    sa.Column('log_index', sa.BigInteger(), nullable=False),
    sa.Column('path', sa.Text(), nullable=False),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.ForeignKeyConstraint(['transaction_hash'], ['mantle_sepolia_transactions.hash'], name=op.f('fk_mantle_sepolia_logs_transaction_hash_mantle_sepolia_transactions'), ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('transaction_hash', 'log_index', name='pk_mantle_sepolia_log_index'),
    sa.UniqueConstraint('transaction_hash', 'log_index', name='uq_mantle_sepolia_log_index_transaction_hash_log_index')
    )
    op.create_index('idx_mantle_sepolia_logs_block_hash_log_index', 'mantle_sepolia_logs', ['block_hash', 'log_index'], unique=True)
    op.create_index(op.f('ix_mantle_sepolia_logs_address'), 'mantle_sepolia_logs', ['address'], unique=False)
    op.create_index(op.f('ix_mantle_sepolia_logs_block_hash'), 'mantle_sepolia_logs', ['block_hash'], unique=False)
    op.create_index(op.f('ix_mantle_sepolia_logs_transaction_hash'), 'mantle_sepolia_logs', ['transaction_hash'], unique=False)
    op.create_table('xai_logs',
    sa.Column('transaction_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('address', sa.LargeBinary(length=20), nullable=False),
    sa.Column('row_id', sa.BigInteger(), nullable=False),
    sa.Column('selector', sa.VARCHAR(length=256), nullable=True),
    sa.Column('topic1', sa.VARCHAR(length=256), nullable=True),
    sa.Column('topic2', sa.VARCHAR(length=256), nullable=True),
    sa.Column('log_index', sa.BigInteger(), nullable=False),
    sa.Column('path', sa.Text(), nullable=False),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.ForeignKeyConstraint(['transaction_hash'], ['xai_transactions.hash'], name=op.f('fk_xai_logs_transaction_hash_xai_transactions'), ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('transaction_hash', 'log_index', name='pk_xai_log_index'),
    sa.UniqueConstraint('transaction_hash', 'log_index', name='uq_xai_log_idx_tx_hash_log_idx')
    )
    op.create_index(op.f('ix_xai_logs_address'), 'xai_logs', ['address'], unique=False)
    op.create_index(op.f('ix_xai_logs_block_hash'), 'xai_logs', ['block_hash'], unique=False)
    op.create_index(op.f('ix_xai_logs_transaction_hash'), 'xai_logs', ['transaction_hash'], unique=False)
    op.create_table('xai_sepolia_logs',
    sa.Column('transaction_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('address', sa.LargeBinary(length=20), nullable=False),
    sa.Column('row_id', sa.BigInteger(), nullable=False),
    sa.Column('selector', sa.VARCHAR(length=256), nullable=True),
    sa.Column('topic1', sa.VARCHAR(length=256), nullable=True),
    sa.Column('topic2', sa.VARCHAR(length=256), nullable=True),
    sa.Column('log_index', sa.BigInteger(), nullable=False),
    sa.Column('path', sa.Text(), nullable=False),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.ForeignKeyConstraint(['transaction_hash'], ['xai_sepolia_transactions.hash'], name=op.f('fk_xai_sepolia_logs_transaction_hash_xai_sepolia_transactions'), ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('transaction_hash', 'log_index', name='pk_xai_sepolia_log_index'),
    sa.UniqueConstraint('transaction_hash', 'log_index', name='uq_xai_sepolia_log_idx_tx_hash_log_idx')
    )
    op.create_index(op.f('ix_xai_sepolia_logs_address'), 'xai_sepolia_logs', ['address'], unique=False)
    op.create_index(op.f('ix_xai_sepolia_logs_block_hash'), 'xai_sepolia_logs', ['block_hash'], unique=False)
    op.create_index(op.f('ix_xai_sepolia_logs_transaction_hash'), 'xai_sepolia_logs', ['transaction_hash'], unique=False)
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f('ix_xai_sepolia_logs_transaction_hash'), table_name='xai_sepolia_logs')
    op.drop_index(op.f('ix_xai_sepolia_logs_block_hash'), table_name='xai_sepolia_logs')
    op.drop_index(op.f('ix_xai_sepolia_logs_address'), table_name='xai_sepolia_logs')
    op.drop_table('xai_sepolia_logs')
    op.drop_index(op.f('ix_xai_logs_transaction_hash'), table_name='xai_logs')
    op.drop_index(op.f('ix_xai_logs_block_hash'), table_name='xai_logs')
    op.drop_index(op.f('ix_xai_logs_address'), table_name='xai_logs')
    op.drop_table('xai_logs')
    op.drop_index(op.f('ix_mantle_sepolia_logs_transaction_hash'), table_name='mantle_sepolia_logs')
    op.drop_index(op.f('ix_mantle_sepolia_logs_block_hash'), table_name='mantle_sepolia_logs')
    op.drop_index(op.f('ix_mantle_sepolia_logs_address'), table_name='mantle_sepolia_logs')
    op.drop_index('idx_mantle_sepolia_logs_block_hash_log_index', table_name='mantle_sepolia_logs')
    op.drop_table('mantle_sepolia_logs')
    op.drop_index(op.f('ix_mantle_logs_transaction_hash'), table_name='mantle_logs')
    op.drop_index(op.f('ix_mantle_logs_block_hash'), table_name='mantle_logs')
    op.drop_index(op.f('ix_mantle_logs_address'), table_name='mantle_logs')
    op.drop_index('idx_mantle_logs_block_hash_log_index', table_name='mantle_logs')
    op.drop_table('mantle_logs')
    op.drop_index(op.f('ix_game7_orbit_arbitrum_sepolia_logs_transaction_hash'), table_name='game7_orbit_arbitrum_sepolia_logs')
    op.drop_index(op.f('ix_game7_orbit_arbitrum_sepolia_logs_block_hash'), table_name='game7_orbit_arbitrum_sepolia_logs')
    op.drop_index(op.f('ix_game7_orbit_arbitrum_sepolia_logs_address'), table_name='game7_orbit_arbitrum_sepolia_logs')
    op.drop_table('game7_orbit_arbitrum_sepolia_logs')
    op.drop_index(op.f('ix_arbitrum_sepolia_logs_transaction_hash'), table_name='arbitrum_sepolia_logs')
    op.drop_index(op.f('ix_arbitrum_sepolia_logs_block_hash'), table_name='arbitrum_sepolia_logs')
    op.drop_index(op.f('ix_arbitrum_sepolia_logs_address'), table_name='arbitrum_sepolia_logs')
    op.drop_table('arbitrum_sepolia_logs')
    op.drop_index(op.f('ix_arbitrum_one_logs_transaction_hash'), table_name='arbitrum_one_logs')
    op.drop_index(op.f('ix_arbitrum_one_logs_block_hash'), table_name='arbitrum_one_logs')
    op.drop_index(op.f('ix_arbitrum_one_logs_address'), table_name='arbitrum_one_logs')
    op.drop_table('arbitrum_one_logs')
    op.drop_index(op.f('ix_xai_transactions_type'), table_name='xai_transactions')
    op.drop_index(op.f('ix_xai_transactions_to_address'), table_name='xai_transactions')
    op.drop_index(op.f('ix_xai_transactions_selector'), table_name='xai_transactions')
    op.drop_index(op.f('ix_xai_transactions_index'), table_name='xai_transactions')
    op.drop_index(op.f('ix_xai_transactions_hash'), table_name='xai_transactions')
    op.drop_index(op.f('ix_xai_transactions_from_address'), table_name='xai_transactions')
    op.drop_index(op.f('ix_xai_transactions_block_number'), table_name='xai_transactions')
    op.drop_index(op.f('ix_xai_transactions_block_hash'), table_name='xai_transactions')
    op.drop_table('xai_transactions')
    op.drop_index(op.f('ix_xai_sepolia_transactions_type'), table_name='xai_sepolia_transactions')
    op.drop_index(op.f('ix_xai_sepolia_transactions_to_address'), table_name='xai_sepolia_transactions')
    op.drop_index(op.f('ix_xai_sepolia_transactions_selector'), table_name='xai_sepolia_transactions')
    op.drop_index(op.f('ix_xai_sepolia_transactions_index'), table_name='xai_sepolia_transactions')
    op.drop_index(op.f('ix_xai_sepolia_transactions_hash'), table_name='xai_sepolia_transactions')
    op.drop_index(op.f('ix_xai_sepolia_transactions_from_address'), table_name='xai_sepolia_transactions')
    op.drop_index(op.f('ix_xai_sepolia_transactions_block_number'), table_name='xai_sepolia_transactions')
    op.drop_index(op.f('ix_xai_sepolia_transactions_block_hash'), table_name='xai_sepolia_transactions')
    op.drop_table('xai_sepolia_transactions')
    op.drop_index(op.f('ix_mantle_transactions_type'), table_name='mantle_transactions')
    op.drop_index(op.f('ix_mantle_transactions_to_address'), table_name='mantle_transactions')
    op.drop_index(op.f('ix_mantle_transactions_selector'), table_name='mantle_transactions')
    op.drop_index(op.f('ix_mantle_transactions_index'), table_name='mantle_transactions')
    op.drop_index(op.f('ix_mantle_transactions_hash'), table_name='mantle_transactions')
    op.drop_index(op.f('ix_mantle_transactions_from_address'), table_name='mantle_transactions')
    op.drop_index(op.f('ix_mantle_transactions_block_number'), table_name='mantle_transactions')
    op.drop_index(op.f('ix_mantle_transactions_block_hash'), table_name='mantle_transactions')
    op.drop_table('mantle_transactions')
    op.drop_index(op.f('ix_mantle_sepolia_transactions_type'), table_name='mantle_sepolia_transactions')
    op.drop_index(op.f('ix_mantle_sepolia_transactions_to_address'), table_name='mantle_sepolia_transactions')
    op.drop_index(op.f('ix_mantle_sepolia_transactions_selector'), table_name='mantle_sepolia_transactions')
    op.drop_index(op.f('ix_mantle_sepolia_transactions_index'), table_name='mantle_sepolia_transactions')
    op.drop_index(op.f('ix_mantle_sepolia_transactions_hash'), table_name='mantle_sepolia_transactions')
    op.drop_index(op.f('ix_mantle_sepolia_transactions_from_address'), table_name='mantle_sepolia_transactions')
    op.drop_index(op.f('ix_mantle_sepolia_transactions_block_number'), table_name='mantle_sepolia_transactions')
    op.drop_index(op.f('ix_mantle_sepolia_transactions_block_hash'), table_name='mantle_sepolia_transactions')
    op.drop_table('mantle_sepolia_transactions')
    op.drop_index(op.f('ix_game7_orbit_arbitrum_sepolia_transactions_type'), table_name='game7_orbit_arbitrum_sepolia_transactions')
    op.drop_index(op.f('ix_game7_orbit_arbitrum_sepolia_transactions_to_address'), table_name='game7_orbit_arbitrum_sepolia_transactions')
    op.drop_index(op.f('ix_game7_orbit_arbitrum_sepolia_transactions_selector'), table_name='game7_orbit_arbitrum_sepolia_transactions')
    op.drop_index(op.f('ix_game7_orbit_arbitrum_sepolia_transactions_index'), table_name='game7_orbit_arbitrum_sepolia_transactions')
    op.drop_index(op.f('ix_game7_orbit_arbitrum_sepolia_transactions_hash'), table_name='game7_orbit_arbitrum_sepolia_transactions')
    op.drop_index(op.f('ix_game7_orbit_arbitrum_sepolia_transactions_from_address'), table_name='game7_orbit_arbitrum_sepolia_transactions')
    op.drop_index(op.f('ix_game7_orbit_arbitrum_sepolia_transactions_block_number'), table_name='game7_orbit_arbitrum_sepolia_transactions')
    op.drop_index(op.f('ix_game7_orbit_arbitrum_sepolia_transactions_block_hash'), table_name='game7_orbit_arbitrum_sepolia_transactions')
    op.drop_table('game7_orbit_arbitrum_sepolia_transactions')
    op.drop_index(op.f('ix_arbitrum_sepolia_transactions_type'), table_name='arbitrum_sepolia_transactions')
    op.drop_index(op.f('ix_arbitrum_sepolia_transactions_to_address'), table_name='arbitrum_sepolia_transactions')
    op.drop_index(op.f('ix_arbitrum_sepolia_transactions_selector'), table_name='arbitrum_sepolia_transactions')
    op.drop_index(op.f('ix_arbitrum_sepolia_transactions_index'), table_name='arbitrum_sepolia_transactions')
    op.drop_index(op.f('ix_arbitrum_sepolia_transactions_hash'), table_name='arbitrum_sepolia_transactions')
    op.drop_index(op.f('ix_arbitrum_sepolia_transactions_from_address'), table_name='arbitrum_sepolia_transactions')
    op.drop_index(op.f('ix_arbitrum_sepolia_transactions_block_number'), table_name='arbitrum_sepolia_transactions')
    op.drop_index(op.f('ix_arbitrum_sepolia_transactions_block_hash'), table_name='arbitrum_sepolia_transactions')
    op.drop_table('arbitrum_sepolia_transactions')
    op.drop_index(op.f('ix_arbitrum_one_transactions_type'), table_name='arbitrum_one_transactions')
    op.drop_index(op.f('ix_arbitrum_one_transactions_to_address'), table_name='arbitrum_one_transactions')
    op.drop_index(op.f('ix_arbitrum_one_transactions_selector'), table_name='arbitrum_one_transactions')
    op.drop_index(op.f('ix_arbitrum_one_transactions_index'), table_name='arbitrum_one_transactions')
    op.drop_index(op.f('ix_arbitrum_one_transactions_hash'), table_name='arbitrum_one_transactions')
    op.drop_index(op.f('ix_arbitrum_one_transactions_from_address'), table_name='arbitrum_one_transactions')
    op.drop_index(op.f('ix_arbitrum_one_transactions_block_number'), table_name='arbitrum_one_transactions')
    op.drop_index(op.f('ix_arbitrum_one_transactions_block_hash'), table_name='arbitrum_one_transactions')
    op.drop_table('arbitrum_one_transactions')
    op.drop_index(op.f('ix_xai_sepolia_reorgs_block_number'), table_name='xai_sepolia_reorgs')
    op.drop_index(op.f('ix_xai_sepolia_reorgs_block_hash'), table_name='xai_sepolia_reorgs')
    op.drop_table('xai_sepolia_reorgs')
    op.drop_index(op.f('ix_xai_sepolia_blocks_block_timestamp'), table_name='xai_sepolia_blocks')
    op.drop_index(op.f('ix_xai_sepolia_blocks_block_number'), table_name='xai_sepolia_blocks')
    op.drop_table('xai_sepolia_blocks')
    op.drop_index(op.f('ix_xai_reorgs_block_number'), table_name='xai_reorgs')
    op.drop_index(op.f('ix_xai_reorgs_block_hash'), table_name='xai_reorgs')
    op.drop_table('xai_reorgs')
    op.drop_index(op.f('ix_xai_blocks_block_timestamp'), table_name='xai_blocks')
    op.drop_index(op.f('ix_xai_blocks_block_number'), table_name='xai_blocks')
    op.drop_table('xai_blocks')
    op.drop_index(op.f('ix_mantle_sepolia_reorgs_block_number'), table_name='mantle_sepolia_reorgs')
    op.drop_index(op.f('ix_mantle_sepolia_reorgs_block_hash'), table_name='mantle_sepolia_reorgs')
    op.drop_table('mantle_sepolia_reorgs')
    op.drop_index(op.f('ix_mantle_sepolia_blocks_block_timestamp'), table_name='mantle_sepolia_blocks')
    op.drop_index(op.f('ix_mantle_sepolia_blocks_block_number'), table_name='mantle_sepolia_blocks')
    op.drop_table('mantle_sepolia_blocks')
    op.drop_index(op.f('ix_mantle_reorgs_block_number'), table_name='mantle_reorgs')
    op.drop_index(op.f('ix_mantle_reorgs_block_hash'), table_name='mantle_reorgs')
    op.drop_table('mantle_reorgs')
    op.drop_index(op.f('ix_mantle_blocks_block_timestamp'), table_name='mantle_blocks')
    op.drop_index(op.f('ix_mantle_blocks_block_number'), table_name='mantle_blocks')
    op.drop_table('mantle_blocks')
    op.drop_index(op.f('ix_game7_orbit_arbitrum_sepolia_reorgs_block_number'), table_name='game7_orbit_arbitrum_sepolia_reorgs')
    op.drop_index(op.f('ix_game7_orbit_arbitrum_sepolia_reorgs_block_hash'), table_name='game7_orbit_arbitrum_sepolia_reorgs')
    op.drop_table('game7_orbit_arbitrum_sepolia_reorgs')
    op.drop_index(op.f('ix_game7_orbit_arbitrum_sepolia_blocks_block_timestamp'), table_name='game7_orbit_arbitrum_sepolia_blocks')
    op.drop_index(op.f('ix_game7_orbit_arbitrum_sepolia_blocks_block_number'), table_name='game7_orbit_arbitrum_sepolia_blocks')
    op.drop_table('game7_orbit_arbitrum_sepolia_blocks')
    op.drop_index(op.f('ix_arbitrum_sepolia_reorgs_block_number'), table_name='arbitrum_sepolia_reorgs')
    op.drop_index(op.f('ix_arbitrum_sepolia_reorgs_block_hash'), table_name='arbitrum_sepolia_reorgs')
    op.drop_table('arbitrum_sepolia_reorgs')
    op.drop_index(op.f('ix_arbitrum_sepolia_blocks_block_timestamp'), table_name='arbitrum_sepolia_blocks')
    op.drop_index(op.f('ix_arbitrum_sepolia_blocks_block_number'), table_name='arbitrum_sepolia_blocks')
    op.drop_table('arbitrum_sepolia_blocks')
    op.drop_index(op.f('ix_arbitrum_one_reorgs_block_number'), table_name='arbitrum_one_reorgs')
    op.drop_index(op.f('ix_arbitrum_one_reorgs_block_hash'), table_name='arbitrum_one_reorgs')
    op.drop_table('arbitrum_one_reorgs')
    op.drop_index(op.f('ix_arbitrum_one_blocks_block_timestamp'), table_name='arbitrum_one_blocks')
    op.drop_index(op.f('ix_arbitrum_one_blocks_block_number'), table_name='arbitrum_one_blocks')
    op.drop_table('arbitrum_one_blocks')
    # ### end Alembic commands ###



"""Add sepolia

Revision ID: a1ead76c0470
Revises: f2c6aa92e5d2
Create Date: 2024-07-22 14:07:22.992523

"""

from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = "a1ead76c0470"
down_revision: Union[str, None] = "f2c6aa92e5d2"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "sepolia_blocks",
        sa.Column("block_number", sa.BigInteger(), nullable=False),
        sa.Column("block_hash", sa.VARCHAR(length=256), nullable=False),
        sa.Column("block_timestamp", sa.BigInteger(), nullable=False),
        sa.Column("parent_hash", sa.VARCHAR(length=256), nullable=False),
        sa.Column("row_id", sa.BigInteger(), nullable=False),
        sa.Column("path", sa.Text(), nullable=False),
        sa.Column(
            "indexed_at",
            sa.DateTime(timezone=True),
            server_default=sa.text("TIMEZONE('utc', statement_timestamp())"),
            nullable=False,
        ),
        sa.PrimaryKeyConstraint("block_number", name=op.f("pk_sepolia_blocks")),
    )
    op.create_index(
        op.f("ix_sepolia_blocks_block_number"),
        "sepolia_blocks",
        ["block_number"],
        unique=False,
    )
    op.create_index(
        op.f("ix_sepolia_blocks_block_timestamp"),
        "sepolia_blocks",
        ["block_timestamp"],
        unique=False,
    )
    op.create_table(
        "sepolia_reorgs",
        sa.Column("id", sa.UUID(), nullable=False),
        sa.Column("block_number", sa.BigInteger(), nullable=False),
        sa.Column("block_hash", sa.VARCHAR(length=256), nullable=False),
        sa.PrimaryKeyConstraint("id", name=op.f("pk_sepolia_reorgs")),
    )
    op.create_index(
        op.f("ix_sepolia_reorgs_block_hash"),
        "sepolia_reorgs",
        ["block_hash"],
        unique=False,
    )
    op.create_index(
        op.f("ix_sepolia_reorgs_block_number"),
        "sepolia_reorgs",
        ["block_number"],
        unique=False,
    )
    op.create_table(
        "sepolia_transactions",
        sa.Column("block_number", sa.BigInteger(), nullable=False),
        sa.Column("hash", sa.VARCHAR(length=256), nullable=False),
        sa.Column("from_address", sa.LargeBinary(length=20), nullable=False),
        sa.Column("to_address", sa.LargeBinary(length=20), nullable=False),
        sa.Column("selector", sa.VARCHAR(length=256), nullable=True),
        sa.Column("type", sa.Integer(), nullable=True),
        sa.Column("row_id", sa.BigInteger(), nullable=False),
        sa.Column("block_hash", sa.VARCHAR(length=256), nullable=False),
        sa.Column("index", sa.BigInteger(), nullable=False),
        sa.Column("path", sa.Text(), nullable=False),
        sa.Column(
            "indexed_at",
            sa.DateTime(timezone=True),
            server_default=sa.text("TIMEZONE('utc', statement_timestamp())"),
            nullable=False,
        ),
        sa.ForeignKeyConstraint(
            ["block_number"],
            ["sepolia_blocks.block_number"],
            name=op.f("fk_sepolia_transactions_block_number_sepolia_blocks"),
            ondelete="CASCADE",
        ),
        sa.PrimaryKeyConstraint("hash", name=op.f("pk_sepolia_transactions")),
    )
    op.create_index(
        op.f("ix_sepolia_transactions_block_hash"),
        "sepolia_transactions",
        ["block_hash"],
        unique=False,
    )
    op.create_index(
        op.f("ix_sepolia_transactions_block_number"),
        "sepolia_transactions",
        ["block_number"],
        unique=False,
    )
    op.create_index(
        op.f("ix_sepolia_transactions_from_address"),
        "sepolia_transactions",
        ["from_address"],
        unique=False,
    )
    op.create_index(
        op.f("ix_sepolia_transactions_hash"),
        "sepolia_transactions",
        ["hash"],
        unique=True,
    )
    op.create_index(
        op.f("ix_sepolia_transactions_index"),
        "sepolia_transactions",
        ["index"],
        unique=False,
    )
    op.create_index(
        op.f("ix_sepolia_transactions_selector"),
        "sepolia_transactions",
        ["selector"],
        unique=False,
    )
    op.create_index(
        op.f("ix_sepolia_transactions_to_address"),
        "sepolia_transactions",
        ["to_address"],
        unique=False,
    )
    op.create_index(
        op.f("ix_sepolia_transactions_type"),
        "sepolia_transactions",
        ["type"],
        unique=False,
    )
    op.create_table(
        "sepolia_logs",
        sa.Column("transaction_hash", sa.VARCHAR(length=256), nullable=False),
        sa.Column("block_hash", sa.VARCHAR(length=256), nullable=False),
        sa.Column("address", sa.LargeBinary(length=20), nullable=False),
        sa.Column("row_id", sa.BigInteger(), nullable=False),
        sa.Column("selector", sa.VARCHAR(length=256), nullable=True),
        sa.Column("topic1", sa.VARCHAR(length=256), nullable=True),
        sa.Column("topic2", sa.VARCHAR(length=256), nullable=True),
        sa.Column("topic3", sa.VARCHAR(length=256), nullable=True),
        sa.Column("log_index", sa.BigInteger(), nullable=False),
        sa.Column("path", sa.Text(), nullable=False),
        sa.Column(
            "indexed_at",
            sa.DateTime(timezone=True),
            server_default=sa.text("TIMEZONE('utc', statement_timestamp())"),
            nullable=False,
        ),
        sa.ForeignKeyConstraint(
            ["transaction_hash"],
            ["sepolia_transactions.hash"],
            name=op.f("fk_sepolia_logs_transaction_hash_sepolia_transactions"),
            ondelete="CASCADE",
        ),
        sa.PrimaryKeyConstraint(
            "transaction_hash", "log_index", name="pk_sepolia_log_index"
        ),
        sa.UniqueConstraint(
            "transaction_hash",
            "log_index",
            name="uq_sepolia_log_index_transaction_hash_log_index",
        ),
    )
    op.create_index(
        "idx_sepolia_logs_address_selector",
        "sepolia_logs",
        ["address", "selector"],
        unique=False,
    )
    op.create_index(
        "idx_sepolia_logs_block_hash_log_index",
        "sepolia_logs",
        ["block_hash", "log_index"],
        unique=True,
    )
    op.create_index(
        op.f("ix_sepolia_logs_address"),
        "sepolia_logs",
        ["address"],
        unique=False,
    )
    op.create_index(
        op.f("ix_sepolia_logs_block_hash"),
        "sepolia_logs",
        ["block_hash"],
        unique=False,
    )
    op.create_index(
        op.f("ix_sepolia_logs_transaction_hash"),
        "sepolia_logs",
        ["transaction_hash"],
        unique=False,
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(
        op.f("ix_sepolia_logs_transaction_hash"),
        table_name="sepolia_logs",
    )
    op.drop_index(op.f("ix_sepolia_logs_block_hash"), table_name="sepolia_logs")
    op.drop_index(op.f("ix_sepolia_logs_address"), table_name="sepolia_logs")
    op.drop_index(
        "idx_sepolia_logs_block_hash_log_index",
        table_name="sepolia_logs",
    )
    op.drop_index("idx_sepolia_logs_address_selector", table_name="sepolia_logs")
    op.drop_table("sepolia_logs")
    op.drop_index(
        op.f("ix_sepolia_transactions_type"),
        table_name="sepolia_transactions",
    )
    op.drop_index(
        op.f("ix_sepolia_transactions_to_address"),
        table_name="sepolia_transactions",
    )
    op.drop_index(
        op.f("ix_sepolia_transactions_selector"),
        table_name="sepolia_transactions",
    )
    op.drop_index(
        op.f("ix_sepolia_transactions_index"),
        table_name="sepolia_transactions",
    )
    op.drop_index(
        op.f("ix_sepolia_transactions_hash"),
        table_name="sepolia_transactions",
    )
    op.drop_index(
        op.f("ix_sepolia_transactions_from_address"),
        table_name="sepolia_transactions",
    )
    op.drop_index(
        op.f("ix_sepolia_transactions_block_number"),
        table_name="sepolia_transactions",
    )
    op.drop_index(
        op.f("ix_sepolia_transactions_block_hash"),
        table_name="sepolia_transactions",
    )
    op.drop_table("sepolia_transactions")
    op.drop_index(
        op.f("ix_sepolia_reorgs_block_number"),
        table_name="sepolia_reorgs",
    )
    op.drop_index(
        op.f("ix_sepolia_reorgs_block_hash"),
        table_name="sepolia_reorgs",
    )
    op.drop_table("sepolia_reorgs")
    op.drop_index(
        op.f("ix_sepolia_blocks_block_timestamp"),
        table_name="sepolia_blocks",
    )
    op.drop_index(
        op.f("ix_sepolia_blocks_block_number"),
        table_name="sepolia_blocks",
    )
    op.drop_table("sepolia_blocks")
    # ### end Alembic commands ###



"""Logs address selector index

Revision ID: e02c90ea67bb
Revises: a4ef4f9031e4
Create Date: 2024-06-06 13:12:14.594600

"""

from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = "e02c90ea67bb"
down_revision: Union[str, None] = "a4ef4f9031e4"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index(
        "idx_arbitrum_one_logs_address_selector",
        "arbitrum_one_logs",
        ["address", "selector"],
        unique=False,
    )
    op.create_index(
        "idx_arbitrum_sepolia_logs_address_selector",
        "arbitrum_sepolia_logs",
        ["address", "selector"],
        unique=False,
    )
    op.create_index(
        "idx_ethereum_logs_address_selector",
        "ethereum_logs",
        ["address", "selector"],
        unique=False,
    )
    op.create_index(
        "idx_game7_orbit_arbitrum_sepolia_logs_address_selector",
        "game7_orbit_arbitrum_sepolia_logs",
        ["address", "selector"],
        unique=False,
    )
    op.create_index(
        "idx_mantle_logs_address_selector",
        "mantle_logs",
        ["address", "selector"],
        unique=False,
    )
    op.create_index(
        "idx_mantle_sepolia_logs_address_selector",
        "mantle_sepolia_logs",
        ["address", "selector"],
        unique=False,
    )
    op.create_index(
        "idx_polygon_logs_address_selector",
        "polygon_logs",
        ["address", "selector"],
        unique=False,
    )
    op.create_index(
        "idx_xai_logs_address_selector",
        "xai_logs",
        ["address", "selector"],
        unique=False,
    )
    op.create_index(
        "idx_xai_sepolia_logs_address_selector",
        "xai_sepolia_logs",
        ["address", "selector"],
        unique=False,
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(
        "idx_xai_sepolia_logs_address_selector", table_name="xai_sepolia_logs"
    )
    op.drop_index("idx_xai_logs_address_selector", table_name="xai_logs")
    op.drop_index("idx_polygon_logs_address_selector", table_name="polygon_logs")
    op.drop_index(
        "idx_mantle_sepolia_logs_address_selector", table_name="mantle_sepolia_logs"
    )
    op.drop_index("idx_mantle_logs_address_selector", table_name="mantle_logs")
    op.drop_index(
        "idx_game7_orbit_arbitrum_sepolia_logs_address_selector",
        table_name="game7_orbit_arbitrum_sepolia_logs",
    )
    op.drop_index("idx_ethereum_logs_address_selector", table_name="ethereum_logs")
    op.drop_index(
        "idx_arbitrum_sepolia_logs_address_selector", table_name="arbitrum_sepolia_logs"
    )
    op.drop_index(
        "idx_arbitrum_one_logs_address_selector", table_name="arbitrum_one_logs"
    )
    # ### end Alembic commands ###



from logging.config import fileConfig

from alembic import context
from sqlalchemy import engine_from_config, pool

# this is the Alembic Config object, which provides
# access to the values within the .ini file in use.
config = context.config

# Interpret the config file for Python logging.
# This line sets up loggers basically.
if config.config_file_name is not None:
    fileConfig(config.config_file_name)

# add your model's MetaData object here
# for 'autogenerate' support
# from myapp import mymodel
# target_metadata = mymodel.Base.metadata
from moonstreamdbv3.models import Base as MoonstreamBase

target_metadata = MoonstreamBase.metadata

# other values from the config, defined by the needs of env.py,
# can be acquired:
# my_important_option = config.get_main_option("my_important_option")
# ... etc.
from moonstreamdbv3.models import (
    AmoyLabel,
    ArbitrumNovaLabel,
    ArbitrumOneLabel,
    ArbitrumSepoliaLabel,
    AvalancheFujiLabel,
    AvalancheLabel,
    BaseLabel,
    BlastLabel,
    BlastSepoliaLabel,
    EthereumLabel,
    Game7OrbitArbitrumSepoliaLabel,
    ImxZkevmLabel,
    ImxZkevmSepoliaLabel,
    MantleLabel,
    MantleSepoliaLabel,
    MumbaiLabel,
    PolygonLabel,
    ProofOfPlayApexLabel,
    SepoliaLabel,
    StarknetLabel,
    StarknetSepoliaLabel,
    XaiLabel,
    XaiSepoliaLabel,
    XDaiLabel,
    ZkSyncEraLabel,
    ZkSyncEraSepoliaLabel,
)


def include_symbol(tablename, schema):
    return tablename in {
        EthereumLabel.__tablename__,
        SepoliaLabel.__tablename__,
        PolygonLabel.__tablename__,
        MumbaiLabel.__tablename__,
        AmoyLabel.__tablename__,
        XDaiLabel.__tablename__,
        ZkSyncEraLabel.__tablename__,
        ZkSyncEraSepoliaLabel.__tablename__,
        BaseLabel.__tablename__,
        ArbitrumNovaLabel.__tablename__,
        ArbitrumOneLabel.__tablename__,
        ArbitrumSepoliaLabel.__tablename__,
        Game7OrbitArbitrumSepoliaLabel.__tablename__,
        XaiLabel.__tablename__,
        XaiSepoliaLabel.__tablename__,
        AvalancheLabel.__tablename__,
        AvalancheFujiLabel.__tablename__,
        BlastLabel.__tablename__,
        BlastSepoliaLabel.__tablename__,
        ProofOfPlayApexLabel.__tablename__,
        StarknetLabel.__tablename__,
        StarknetSepoliaLabel.__tablename__,
        MantleLabel.__tablename__,
        MantleSepoliaLabel.__tablename__,
        ImxZkevmLabel.__tablename__,
        ImxZkevmSepoliaLabel.__tablename__,
    }


def run_migrations_offline() -> None:
    """Run migrations in 'offline' mode.

    This configures the context with just a URL
    and not an Engine, though an Engine is acceptable
    here as well.  By skipping the Engine creation
    we don't even need a DBAPI to be available.

    Calls to context.execute() here emit the given string to the
    script output.

    """
    url = config.get_main_option("sqlalchemy.url")
    context.configure(
        url=url,
        target_metadata=target_metadata,
        literal_binds=True,
        dialect_opts={"paramstyle": "named"},
        version_table="alembic_version",
        include_schemas=True,
        include_symbol=include_symbol,
    )

    with context.begin_transaction():
        context.run_migrations()


def run_migrations_online() -> None:
    """Run migrations in 'online' mode.

    In this scenario we need to create an Engine
    and associate a connection with the context.

    """
    connectable = engine_from_config(
        config.get_section(config.config_ini_section, {}),
        prefix="sqlalchemy.",
        poolclass=pool.NullPool,
    )

    with connectable.connect() as connection:
        context.configure(
            connection=connection,
            target_metadata=target_metadata,
            version_table="alembic_version",
            include_schemas=True,
            include_symbol=include_symbol,
        )

        with context.begin_transaction():
            context.run_migrations()


if context.is_offline_mode():
    run_migrations_offline()
else:
    run_migrations_online()



"""Bytea address in labels

Revision ID: 2e90b758090c
Revises: 792ca9c4722c
Create Date: 2024-07-15 17:38:36.529835

"""

from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = "2e90b758090c"
down_revision: Union[str, None] = "9ca39b11e12a"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.execute(
        "ALTER TABLE amoy_labels ALTER COLUMN caller_address TYPE bytea USING caller_address::bytea"
    )
    op.execute(
        "ALTER TABLE amoy_labels ALTER COLUMN origin_address TYPE bytea USING origin_address::bytea"
    )
    op.execute(
        "ALTER TABLE amoy_labels ALTER COLUMN address TYPE bytea USING address::bytea"
    )
    op.execute(
        "ALTER TABLE arbitrum_nova_labels ALTER COLUMN caller_address TYPE bytea USING caller_address::bytea"
    )
    op.execute(
        "ALTER TABLE arbitrum_nova_labels ALTER COLUMN origin_address TYPE bytea USING origin_address::bytea"
    )
    op.execute(
        "ALTER TABLE arbitrum_nova_labels ALTER COLUMN address TYPE bytea USING address::bytea"
    )
    op.execute(
        "ALTER TABLE arbitrum_one_labels ALTER COLUMN caller_address TYPE bytea USING caller_address::bytea"
    )
    op.execute(
        "ALTER TABLE arbitrum_one_labels ALTER COLUMN origin_address TYPE bytea USING origin_address::bytea"
    )
    op.execute(
        "ALTER TABLE arbitrum_one_labels ALTER COLUMN address TYPE bytea USING address::bytea"
    )
    op.execute(
        "ALTER TABLE arbitrum_sepolia_labels ALTER COLUMN caller_address TYPE bytea USING caller_address::bytea"
    )
    op.execute(
        "ALTER TABLE arbitrum_sepolia_labels ALTER COLUMN origin_address TYPE bytea USING origin_address::bytea"
    )
    op.execute(
        "ALTER TABLE arbitrum_sepolia_labels ALTER COLUMN address TYPE bytea USING address::bytea"
    )
    op.execute(
        "ALTER TABLE avalanche_fuji_labels ALTER COLUMN caller_address TYPE bytea USING caller_address::bytea"
    )
    op.execute(
        "ALTER TABLE avalanche_fuji_labels ALTER COLUMN origin_address TYPE bytea USING origin_address::bytea"
    )
    op.execute(
        "ALTER TABLE avalanche_fuji_labels ALTER COLUMN address TYPE bytea USING address::bytea"
    )
    op.execute(
        "ALTER TABLE avalanche_labels ALTER COLUMN caller_address TYPE bytea USING caller_address::bytea"
    )
    op.execute(
        "ALTER TABLE avalanche_labels ALTER COLUMN origin_address TYPE bytea USING origin_address::bytea"
    )
    op.execute(
        "ALTER TABLE avalanche_labels ALTER COLUMN address TYPE bytea USING address::bytea"
    )
    op.execute(
        "ALTER TABLE base_labels ALTER COLUMN caller_address TYPE bytea USING caller_address::bytea"
    )
    op.execute(
        "ALTER TABLE base_labels ALTER COLUMN origin_address TYPE bytea USING origin_address::bytea"
    )
    op.execute(
        "ALTER TABLE base_labels ALTER COLUMN address TYPE bytea USING address::bytea"
    )
    op.execute(
        "ALTER TABLE blast_labels ALTER COLUMN caller_address TYPE bytea USING caller_address::bytea"
    )
    op.execute(
        "ALTER TABLE blast_labels ALTER COLUMN origin_address TYPE bytea USING origin_address::bytea"
    )
    op.execute(
        "ALTER TABLE blast_labels ALTER COLUMN address TYPE bytea USING address::bytea"
    )
    op.execute(
        "ALTER TABLE blast_sepolia_labels ALTER COLUMN caller_address TYPE bytea USING caller_address::bytea"
    )
    op.execute(
        "ALTER TABLE blast_sepolia_labels ALTER COLUMN origin_address TYPE bytea USING origin_address::bytea"
    )
    op.execute(
        "ALTER TABLE blast_sepolia_labels ALTER COLUMN address TYPE bytea USING address::bytea"
    )
    op.execute(
        "ALTER TABLE ethereum_labels ALTER COLUMN caller_address TYPE bytea USING caller_address::bytea"
    )
    op.execute(
        "ALTER TABLE ethereum_labels ALTER COLUMN origin_address TYPE bytea USING origin_address::bytea"
    )
    op.execute(
        "ALTER TABLE ethereum_labels ALTER COLUMN address TYPE bytea USING address::bytea"
    )
    op.execute(
        "ALTER TABLE game7_orbit_arbitrum_sepolia_labels ALTER COLUMN caller_address TYPE bytea USING caller_address::bytea"
    )
    op.execute(
        "ALTER TABLE game7_orbit_arbitrum_sepolia_labels ALTER COLUMN origin_address TYPE bytea USING origin_address::bytea"
    )
    op.execute(
        "ALTER TABLE game7_orbit_arbitrum_sepolia_labels ALTER COLUMN address TYPE bytea USING address::bytea"
    )
    op.execute(
        "ALTER TABLE mantle_labels ALTER COLUMN caller_address TYPE bytea USING caller_address::bytea"
    )
    op.execute(
        "ALTER TABLE mantle_labels ALTER COLUMN origin_address TYPE bytea USING origin_address::bytea"
    )
    op.execute(
        "ALTER TABLE mantle_labels ALTER COLUMN address TYPE bytea USING address::bytea"
    )
    op.execute(
        "ALTER TABLE mantle_sepolia_labels ALTER COLUMN caller_address TYPE bytea USING caller_address::bytea"
    )
    op.execute(
        "ALTER TABLE mantle_sepolia_labels ALTER COLUMN origin_address TYPE bytea USING origin_address::bytea"
    )
    op.execute(
        "ALTER TABLE mantle_sepolia_labels ALTER COLUMN address TYPE bytea USING address::bytea"
    )
    op.execute(
        "ALTER TABLE mumbai_labels ALTER COLUMN caller_address TYPE bytea USING caller_address::bytea"
    )
    op.execute(
        "ALTER TABLE mumbai_labels ALTER COLUMN origin_address TYPE bytea USING origin_address::bytea"
    )
    op.execute(
        "ALTER TABLE mumbai_labels ALTER COLUMN address TYPE bytea USING address::bytea"
    )
    op.execute(
        "ALTER TABLE polygon_labels ALTER COLUMN caller_address TYPE bytea USING caller_address::bytea"
    )
    op.execute(
        "ALTER TABLE polygon_labels ALTER COLUMN origin_address TYPE bytea USING origin_address::bytea"
    )
    op.execute(
        "ALTER TABLE polygon_labels ALTER COLUMN address TYPE bytea USING address::bytea"
    )
    op.execute(
        "ALTER TABLE proofofplay_apex_labels ALTER COLUMN caller_address TYPE bytea USING caller_address::bytea"
    )
    op.execute(
        "ALTER TABLE proofofplay_apex_labels ALTER COLUMN origin_address TYPE bytea USING origin_address::bytea"
    )
    op.execute(
        "ALTER TABLE proofofplay_apex_labels ALTER COLUMN address TYPE bytea USING address::bytea"
    )
    op.execute(
        "ALTER TABLE sepolia_labels ALTER COLUMN caller_address TYPE bytea USING caller_address::bytea"
    )
    op.execute(
        "ALTER TABLE sepolia_labels ALTER COLUMN origin_address TYPE bytea USING origin_address::bytea"
    )
    op.execute(
        "ALTER TABLE sepolia_labels ALTER COLUMN address TYPE bytea USING address::bytea"
    )
    op.execute(
        "ALTER TABLE starknet_labels ALTER COLUMN caller_address TYPE bytea USING caller_address::bytea"
    )
    op.execute(
        "ALTER TABLE starknet_labels ALTER COLUMN origin_address TYPE bytea USING origin_address::bytea"
    )
    op.execute(
        "ALTER TABLE starknet_labels ALTER COLUMN address TYPE bytea USING address::bytea"
    )
    op.execute(
        "ALTER TABLE starknet_sepolia_labels ALTER COLUMN caller_address TYPE bytea USING caller_address::bytea"
    )
    op.execute(
        "ALTER TABLE starknet_sepolia_labels ALTER COLUMN origin_address TYPE bytea USING origin_address::bytea"
    )
    op.execute(
        "ALTER TABLE starknet_sepolia_labels ALTER COLUMN address TYPE bytea USING address::bytea"
    )
    op.execute(
        "ALTER TABLE xai_labels ALTER COLUMN caller_address TYPE bytea USING caller_address::bytea"
    )
    op.execute(
        "ALTER TABLE xai_labels ALTER COLUMN origin_address TYPE bytea USING origin_address::bytea"
    )
    op.execute(
        "ALTER TABLE xai_labels ALTER COLUMN address TYPE bytea USING address::bytea"
    )
    op.execute(
        "ALTER TABLE xai_sepolia_labels ALTER COLUMN caller_address TYPE bytea USING caller_address::bytea"
    )
    op.execute(
        "ALTER TABLE xai_sepolia_labels ALTER COLUMN origin_address TYPE bytea USING origin_address::bytea"
    )
    op.execute(
        "ALTER TABLE xai_sepolia_labels ALTER COLUMN address TYPE bytea USING address::bytea"
    )
    op.execute(
        "ALTER TABLE xdai_labels ALTER COLUMN caller_address TYPE bytea USING caller_address::bytea"
    )
    op.execute(
        "ALTER TABLE xdai_labels ALTER COLUMN origin_address TYPE bytea USING origin_address::bytea"
    )
    op.execute(
        "ALTER TABLE xdai_labels ALTER COLUMN address TYPE bytea USING address::bytea"
    )
    op.execute(
        "ALTER TABLE zksync_era_labels ALTER COLUMN caller_address TYPE bytea USING caller_address::bytea"
    )
    op.execute(
        "ALTER TABLE zksync_era_labels ALTER COLUMN origin_address TYPE bytea USING origin_address::bytea"
    )
    op.execute(
        "ALTER TABLE zksync_era_labels ALTER COLUMN address TYPE bytea USING address::bytea"
    )
    op.execute(
        "ALTER TABLE zksync_era_sepolia_labels ALTER COLUMN caller_address TYPE bytea USING caller_address::bytea"
    )
    op.execute(
        "ALTER TABLE zksync_era_sepolia_labels ALTER COLUMN origin_address TYPE bytea USING origin_address::bytea"
    )
    op.execute(
        "ALTER TABLE zksync_era_sepolia_labels ALTER COLUMN address TYPE bytea USING address::bytea"
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column(
        "zksync_era_sepolia_labels",
        "address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        nullable=True,
    )
    op.alter_column(
        "zksync_era_sepolia_labels",
        "origin_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "zksync_era_sepolia_labels",
        "caller_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "zksync_era_labels",
        "address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        nullable=True,
    )
    op.alter_column(
        "zksync_era_labels",
        "origin_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "zksync_era_labels",
        "caller_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "xdai_labels",
        "address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        nullable=True,
    )
    op.alter_column(
        "xdai_labels",
        "origin_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "xdai_labels",
        "caller_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "xai_sepolia_labels",
        "address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        nullable=True,
    )
    op.alter_column(
        "xai_sepolia_labels",
        "origin_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "xai_sepolia_labels",
        "caller_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "xai_labels",
        "address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        nullable=True,
    )
    op.alter_column(
        "xai_labels",
        "origin_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "xai_labels",
        "caller_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "starknet_sepolia_labels",
        "address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        nullable=True,
    )
    op.alter_column(
        "starknet_sepolia_labels",
        "origin_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "starknet_sepolia_labels",
        "caller_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "starknet_labels",
        "address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        nullable=True,
    )
    op.alter_column(
        "starknet_labels",
        "origin_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "starknet_labels",
        "caller_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "sepolia_labels",
        "address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        nullable=True,
    )
    op.alter_column(
        "sepolia_labels",
        "origin_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "sepolia_labels",
        "caller_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "proofofplay_apex_labels",
        "address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        nullable=True,
    )
    op.alter_column(
        "proofofplay_apex_labels",
        "origin_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "proofofplay_apex_labels",
        "caller_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "polygon_labels",
        "address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        nullable=True,
    )
    op.alter_column(
        "polygon_labels",
        "origin_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "polygon_labels",
        "caller_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "mumbai_labels",
        "address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        nullable=True,
    )
    op.alter_column(
        "mumbai_labels",
        "origin_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "mumbai_labels",
        "caller_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "mantle_sepolia_labels",
        "address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        nullable=True,
    )
    op.alter_column(
        "mantle_sepolia_labels",
        "origin_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "mantle_sepolia_labels",
        "caller_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "mantle_labels",
        "address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        nullable=True,
    )
    op.alter_column(
        "mantle_labels",
        "origin_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "mantle_labels",
        "caller_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "game7_orbit_arbitrum_sepolia_labels",
        "address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        nullable=True,
    )
    op.alter_column(
        "game7_orbit_arbitrum_sepolia_labels",
        "origin_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "game7_orbit_arbitrum_sepolia_labels",
        "caller_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "ethereum_labels",
        "address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        nullable=True,
    )
    op.alter_column(
        "ethereum_labels",
        "origin_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "ethereum_labels",
        "caller_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "blast_sepolia_labels",
        "address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        nullable=True,
    )
    op.alter_column(
        "blast_sepolia_labels",
        "origin_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "blast_sepolia_labels",
        "caller_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "blast_labels",
        "address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        nullable=True,
    )
    op.alter_column(
        "blast_labels",
        "origin_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "blast_labels",
        "caller_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "base_labels",
        "address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        nullable=True,
    )
    op.alter_column(
        "base_labels",
        "origin_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "base_labels",
        "caller_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "avalanche_labels",
        "address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        nullable=True,
    )
    op.alter_column(
        "avalanche_labels",
        "origin_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "avalanche_labels",
        "caller_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "avalanche_fuji_labels",
        "address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        nullable=True,
    )
    op.alter_column(
        "avalanche_fuji_labels",
        "origin_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "avalanche_fuji_labels",
        "caller_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "arbitrum_sepolia_labels",
        "address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        nullable=True,
    )
    op.alter_column(
        "arbitrum_sepolia_labels",
        "origin_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "arbitrum_sepolia_labels",
        "caller_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "arbitrum_one_labels",
        "address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        nullable=True,
    )
    op.alter_column(
        "arbitrum_one_labels",
        "origin_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "arbitrum_one_labels",
        "caller_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "arbitrum_nova_labels",
        "address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        nullable=True,
    )
    op.alter_column(
        "arbitrum_nova_labels",
        "origin_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "arbitrum_nova_labels",
        "caller_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "amoy_labels",
        "address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        nullable=True,
    )
    op.alter_column(
        "amoy_labels",
        "origin_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    op.alter_column(
        "amoy_labels",
        "caller_address",
        existing_type=sa.LargeBinary(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=True,
    )
    # ### end Alembic commands ###



"""
Moonstream database connection.
"""

import logging
import os
from contextlib import contextmanager
from typing import Generator, Optional

from sqlalchemy import Engine, create_engine
from sqlalchemy.orm import Session, sessionmaker

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

MOONSTREAM_DB_V3_URI = os.environ.get("MOONSTREAM_DB_V3_URI")
if MOONSTREAM_DB_V3_URI is None:
    logger.warning("MOONSTREAM_DB_V3_URI environment variable must be set")

MOONSTREAM_DB_V3_URI_READ_ONLY = os.environ.get("MOONSTREAM_DB_V3_URI_READ_ONLY")
if MOONSTREAM_DB_V3_URI_READ_ONLY is None:
    logger.warning("MOONSTREAM_DB_V3_URI_READ_ONLY environment variable must be set")

try:
    MOONSTREAM_DB_V3_INDEXES_URI = os.environ.get("MOONSTREAM_DB_V3_INDEXES_URI")
    if MOONSTREAM_DB_V3_INDEXES_URI is None:
        raise Warning("MOONSTREAM_DB_V3_INDEXES_URI environment variable must be set")

    MOONSTREAM_DB_V3_INDEXES_URI_READ_ONLY = os.environ.get(
        "MOONSTREAM_DB_V3_INDEXES_URI_READ_ONLY"
    )
    if MOONSTREAM_DB_V3_INDEXES_URI_READ_ONLY is None:
        raise Warning(
            "MOONSTREAM_DB_V3_INDEXES_URI_READ_ONLY environment variable must be set"
        )
except ValueError as e:
    raise ValueError(e)
except Warning:
    logger.warning("Indexes database variables not set")

MOONSTREAM_POOL_SIZE_RAW = os.environ.get("MOONSTREAM_POOL_SIZE")
MOONSTREAM_POOL_SIZE = 1
try:
    if MOONSTREAM_POOL_SIZE_RAW is not None:
        MOONSTREAM_POOL_SIZE = int(MOONSTREAM_POOL_SIZE_RAW)
except:
    raise ValueError(
        f"Could not parse MOONSTREAM_POOL_SIZE as int: {MOONSTREAM_POOL_SIZE_RAW}"
    )

MOONSTREAM_DB_STATEMENT_TIMEOUT_MILLIS_RAW = os.environ.get(
    "MOONSTREAM_DB_STATEMENT_TIMEOUT_MILLIS"
)
MOONSTREAM_DB_STATEMENT_TIMEOUT_MILLIS = 30000
try:
    if MOONSTREAM_DB_STATEMENT_TIMEOUT_MILLIS_RAW is not None:
        MOONSTREAM_DB_STATEMENT_TIMEOUT_MILLIS = int(
            MOONSTREAM_DB_STATEMENT_TIMEOUT_MILLIS_RAW
        )
except:
    raise ValueError(
        f"MOONSTREAM_DB_STATEMENT_TIMEOUT_MILLIS must be an integer: {MOONSTREAM_DB_STATEMENT_TIMEOUT_MILLIS_RAW}"
    )


def create_moonstream_engine(
    url: str,
    pool_size: int,
    statement_timeout: int,
    pool_pre_ping: bool = False,
    schema: Optional[str] = None,
) -> Engine:
    # Pooling: https://docs.sqlalchemy.org/en/14/core/pooling.html#sqlalchemy.pool.QueuePool
    # Statement timeout: https://stackoverflow.com/a/44936982
    options = f"-c statement_timeout={statement_timeout}"
    if schema is not None:
        options += f" -c search_path={schema}"
    return create_engine(
        url=url,
        pool_pre_ping=pool_pre_ping,
        pool_size=pool_size,
        connect_args={"options": options},
    )


class DBEngine:
    def __init__(self, url: str, schema: Optional[str] = None) -> None:
        self._engine = create_moonstream_engine(
            url=url,  # type: ignore
            pool_size=MOONSTREAM_POOL_SIZE,
            statement_timeout=MOONSTREAM_DB_STATEMENT_TIMEOUT_MILLIS,
            schema=schema,
        )

    @property
    def engine(self) -> Engine:
        return self._engine


class MoonstreamDBEngine(DBEngine):
    def __init__(self, schema: Optional[str] = None) -> None:
        super().__init__(url=MOONSTREAM_DB_V3_URI, schema=schema)

        self._session_local = sessionmaker(bind=self.engine)

        self._yield_db_session_ctx = contextmanager(self.yield_db_session)

    @property
    def session_local(self):
        return self._session_local

    @property
    def yield_db_session_ctx(self):
        return self._yield_db_session_ctx

    def yield_db_session(
        self,
    ) -> Generator[Session, None, None]:
        """
        Yields a database connection (created using environment variables).
        As per FastAPI docs:
        https://fastapi.tiangolo.com/tutorial/sql-databases/#create-a-dependency
        """
        session = self._session_local()
        try:
            yield session
        finally:
            session.close()


class MoonstreamDBEngineRO(DBEngine):
    def __init__(self, schema: Optional[str] = None) -> None:
        super().__init__(url=MOONSTREAM_DB_V3_URI_READ_ONLY, schema=schema)

    @property
    def engine(self):
        raise AttributeError(
            "RO_engine should be used instead of engine for read-only access."
        )

    @property
    def RO_engine(self) -> Engine:
        return self._engine

    @property
    def RO_session_local(self):
        return self._session_local

    @property
    def RO_yield_db_session_ctx(self):
        return self._yield_db_session_ctx

    def yield_db_read_only_session(self) -> Generator[Session, None, None]:
        """
        Yields read-only database connection (created using environment variables).
        As per FastAPI docs:
        https://fastapi.tiangolo.com/tutorial/sql-databases/#create-a-dependency
        """
        session = self._session_local()
        try:
            yield session
        finally:
            session.close()


class MoonstreamCustomDBEngine(DBEngine):
    def __init__(self, url: str, schema: Optional[str] = None) -> None:
        super().__init__(url=url, schema=schema)

        logger.warning("Initialized custom database engine with specified URI")

        self._session_local = sessionmaker(bind=self.engine)

        self._yield_db_session_ctx = contextmanager(self.yield_db_session)

    @property
    def session_local(self):
        return self._session_local

    @property
    def yield_db_session_ctx(self):
        return self._yield_db_session_ctx

    def yield_db_session(
        self,
    ) -> Generator[Session, None, None]:
        """
        Yields a database connection (created using environment variables).
        As per FastAPI docs:
        https://fastapi.tiangolo.com/tutorial/sql-databases/#create-a-dependency
        """
        session = self._session_local()
        try:
            yield session
        finally:
            session.close()


class MoonstreamDBIndexesEngine(DBEngine):
    def __init__(self, schema: Optional[str] = None) -> None:
        super().__init__(url=MOONSTREAM_DB_V3_INDEXES_URI, schema=schema)

        self._session_local = sessionmaker(bind=self.engine)

        self._yield_db_session_ctx = contextmanager(self.yield_db_session)

    @property
    def session_local(self):
        return self._session_local

    @property
    def yield_db_session_ctx(self):
        return self._yield_db_session_ctx

    def yield_db_session(
        self,
    ) -> Generator[Session, None, None]:
        """
        Yields a database connection (created using environment variables).
        As per FastAPI docs:
        https://fastapi.tiangolo.com/tutorial/sql-databases/#create-a-dependency
        """
        session = self._session_local()
        try:
            yield session
        finally:
            session.close()


class MoonstreamDBIndexesEngineRO(DBEngine):
    def __init__(self, schema: Optional[str] = None) -> None:
        super().__init__(url=MOONSTREAM_DB_V3_INDEXES_URI_READ_ONLY, schema=schema)

    @property
    def engine(self):
        raise AttributeError(
            "RO_engine should be used instead of engine for read-only access."
        )

    @property
    def RO_engine(self) -> Engine:
        return self._engine

    @property
    def RO_session_local(self):
        return self._session_local

    @property
    def RO_yield_db_session_ctx(self):
        return self._yield_db_session_ctx

    def yield_db_read_only_session(self) -> Generator[Session, None, None]:
        """
        Yields read-only database connection (created using environment variables).
        As per FastAPI docs:
        https://fastapi.tiangolo.com/tutorial/sql-databases/#create-a-dependency
        """
        session = self._session_local()
        try:
            yield session
        finally:
            session.close()



"""Add topic 3

Revision ID: 27086791044c
Revises: e02c90ea67bb
Create Date: 2024-07-11 13:15:45.273275

"""

from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = "27086791044c"
down_revision: Union[str, None] = "e02c90ea67bb"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "arbitrum_one_logs", sa.Column("topic3", sa.VARCHAR(length=256), nullable=True)
    )
    op.add_column(
        "arbitrum_sepolia_logs",
        sa.Column("topic3", sa.VARCHAR(length=256), nullable=True),
    )
    op.add_column(
        "ethereum_logs", sa.Column("topic3", sa.VARCHAR(length=256), nullable=True)
    )
    op.add_column(
        "game7_orbit_arbitrum_sepolia_logs",
        sa.Column("topic3", sa.VARCHAR(length=256), nullable=True),
    )
    op.add_column(
        "mantle_logs", sa.Column("topic3", sa.VARCHAR(length=256), nullable=True)
    )
    op.add_column(
        "mantle_sepolia_logs",
        sa.Column("topic3", sa.VARCHAR(length=256), nullable=True),
    )
    op.add_column(
        "polygon_logs", sa.Column("topic3", sa.VARCHAR(length=256), nullable=True)
    )
    op.add_column(
        "xai_logs", sa.Column("topic3", sa.VARCHAR(length=256), nullable=True)
    )
    op.add_column(
        "xai_sepolia_logs", sa.Column("topic3", sa.VARCHAR(length=256), nullable=True)
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("xai_sepolia_logs", "topic3")
    op.drop_column("xai_logs", "topic3")
    op.drop_column("polygon_logs", "topic3")
    op.drop_column("mantle_sepolia_logs", "topic3")
    op.drop_column("mantle_logs", "topic3")
    op.drop_column("game7_orbit_arbitrum_sepolia_logs", "topic3")
    op.drop_column("ethereum_logs", "topic3")
    op.drop_column("arbitrum_sepolia_logs", "topic3")
    op.drop_column("arbitrum_one_logs", "topic3")
    # ### end Alembic commands ###



"""
Moonstream database V3

Example of label_data column record:
    {
        "label": "ERC20",
        "label_data": {
            "name": "Uniswap",
            "symbol": "UNI"
        }
    },
    {
        "label": "Exchange"
        "label_data": {...}
    }
"""

import uuid

from sqlalchemy import (
    VARCHAR,
    BigInteger,
    Column,
    DateTime,
    Index,
    Integer,
    LargeBinary,
    MetaData,
    Text,
)
from sqlalchemy.dialects.postgresql import JSONB, UUID
from sqlalchemy.ext.compiler import compiles
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.sql import expression, text

"""
Naming conventions doc
https://docs.sqlalchemy.org/en/20/core/constraints.html#configuring-constraint-naming-conventions
"""
convention = {
    "ix": "ix_%(column_0_label)s",
    "uq": "uq_%(table_name)s_%(column_0_name)s",
    "ck": "ck_%(table_name)s_%(constraint_name)s",
    "fk": "fk_%(table_name)s_%(column_0_name)s_%(referred_table_name)s",
    "pk": "pk_%(table_name)s",
}
metadata = MetaData(naming_convention=convention)
Base = declarative_base(metadata=metadata)

"""
Creating a utcnow function which runs on the Posgres database server when created_at and updated_at
fields are populated.
Following:
1. https://docs.sqlalchemy.org/en/13/core/compiler.html#utc-timestamp-function
2. https://www.postgresql.org/docs/current/functions-datetime.html#FUNCTIONS-DATETIME-CURRENT
3. https://stackoverflow.com/a/33532154/13659585
"""


class utcnow(expression.FunctionElement):
    type = DateTime  # type: ignore


@compiles(utcnow, "postgresql")
def pg_utcnow(element, compiler, **kwargs):
    return "TIMEZONE('utc', statement_timestamp())"


class EvmBasedLabel(Base):  # type: ignore
    __abstract__ = True

    id = Column(
        UUID(as_uuid=True),
        primary_key=True,
        default=uuid.uuid4,
        unique=True,
        nullable=False,
    )
    label = Column(VARCHAR(256), nullable=False, index=True)

    transaction_hash = Column(
        VARCHAR(128),
        nullable=False,
        index=True,
    )
    log_index = Column(Integer, nullable=True)

    block_number = Column(
        BigInteger,
        nullable=False,
        index=True,
    )
    block_hash = Column(VARCHAR(256), nullable=False)
    block_timestamp = Column(BigInteger, nullable=False)

    caller_address = Column(
        LargeBinary,
        nullable=True,
        index=True,
    )
    origin_address = Column(
        LargeBinary,
        nullable=True,
        index=True,
    )

    address = Column(
        LargeBinary,
        nullable=False,
        index=True,
    )

    label_name = Column(Text, nullable=True, index=True)
    label_type = Column(VARCHAR(64), nullable=True, index=True)
    label_data = Column(JSONB, nullable=True)

    created_at = Column(
        DateTime(timezone=True), server_default=utcnow(), nullable=False
    )


class EthereumLabel(EvmBasedLabel):  # type: ignore
    __tablename__ = "ethereum_labels"

    __table_args__ = (
        Index(
            "ix_ethereum_labels_addr_block_num",
            "address",
            "block_number",
            unique=False,
        ),
        Index(
            "ix_ethereum_labels_addr_block_ts",
            "address",
            "block_timestamp",
            unique=False,
        ),
        Index(
            "uk_ethereum_labels_tx_hash_tx_call",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer' and label_type='tx_call'"),
        ),
        Index(
            "uk_ethereum_labels_tx_hash_log_idx_evt",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer' and label_type='event'"),
        ),
        Index(
            "uk_ethereum_labels_tx_hash_tx_call_raw",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='tx_call'"),
        ),
        Index(
            "uk_ethereum_labels_tx_hash_log_idx_evt_raw",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='event'"),
        ),
    )


class SepoliaLabel(EvmBasedLabel):  # type: ignore
    __tablename__ = "sepolia_labels"

    __table_args__ = (
        Index(
            "ix_sepolia_labels_addr_block_num",
            "address",
            "block_number",
            unique=False,
        ),
        Index(
            "ix_sepolia_labels_addr_block_ts",
            "address",
            "block_timestamp",
            unique=False,
        ),
        Index(
            "uk_sepolia_labels_tx_hash_tx_call",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer' and label_type='tx_call'"),
        ),
        Index(
            "uk_sepolia_labels_tx_hash_log_idx_evt",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer' and label_type='event'"),
        ),
        Index(
            "uk_sepolia_labels_tx_hash_tx_call_raw",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='tx_call'"),
        ),
        Index(
            "uk_sepolia_labels_tx_hash_log_idx_evt_raw",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='event'"),
        ),
    )


class PolygonLabel(EvmBasedLabel):  # type: ignore
    __tablename__ = "polygon_labels"

    __table_args__ = (
        Index(
            "ix_polygon_labels_addr_block_num",
            "address",
            "block_number",
            unique=False,
        ),
        Index(
            "ix_polygon_labels_addr_block_ts",
            "address",
            "block_timestamp",
            unique=False,
        ),
        Index(
            "uk_polygon_labels_tx_hash_tx_call",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer' and label_type='tx_call'"),
        ),
        Index(
            "uk_polygon_labels_tx_hash_log_idx_evt",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer' and label_type='event'"),
        ),
        Index(
            "uk_polygon_labels_tx_hash_tx_call_raw",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='tx_call'"),
        ),
        Index(
            "uk_polygon_labels_tx_hash_log_idx_evt_raw",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='event'"),
        ),
    )


class MumbaiLabel(EvmBasedLabel):  # type: ignore
    __tablename__ = "mumbai_labels"

    __table_args__ = (
        Index(
            "ix_mumbai_labels_addr_block_num",
            "address",
            "block_number",
            unique=False,
        ),
        Index(
            "ix_mumbai_labels_addr_block_ts",
            "address",
            "block_timestamp",
            unique=False,
        ),
        Index(
            "uk_mumbai_labels_tx_hash_tx_call",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer' and label_type='tx_call'"),
        ),
        Index(
            "uk_mumbai_labels_tx_hash_log_idx_evt",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer' and label_type='event'"),
        ),
        Index(
            "uk_mumbai_labels_tx_hash_tx_call_raw",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='tx_call'"),
        ),
        Index(
            "uk_mumbai_labels_tx_hash_log_idx_evt_raw",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='event'"),
        ),
    )


class AmoyLabel(EvmBasedLabel):  # type: ignore
    __tablename__ = "amoy_labels"

    __table_args__ = (
        Index(
            "ix_amoy_labels_addr_block_num",
            "address",
            "block_number",
            unique=False,
        ),
        Index(
            "ix_amoy_labels_addr_block_ts",
            "address",
            "block_timestamp",
            unique=False,
        ),
        Index(
            "uk_amoy_labels_tx_hash_tx_call",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer' and label_type='tx_call'"),
        ),
        Index(
            "uk_amoy_labels_tx_hash_log_idx_evt",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer' and label_type='event'"),
        ),
        Index(
            "uk_amoy_labels_tx_hash_tx_call_raw",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='tx_call'"),
        ),
        Index(
            "uk_amoy_labels_tx_hash_log_idx_evt_raw",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='event'"),
        ),
    )


class XDaiLabel(EvmBasedLabel):  # type: ignore
    __tablename__ = "xdai_labels"

    __table_args__ = (
        Index(
            "ix_xdai_labels_addr_block_num",
            "address",
            "block_number",
            unique=False,
        ),
        Index(
            "ix_xdai_labels_addr_block_ts",
            "address",
            "block_timestamp",
            unique=False,
        ),
        Index(
            "uk_xdai_labels_tx_hash_tx_call",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer' and label_type='tx_call'"),
        ),
        Index(
            "uk_xdai_labels_tx_hash_log_idx_evt",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer' and label_type='event'"),
        ),
        Index(
            "uk_xdai_labels_tx_hash_tx_call_raw",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='tx_call'"),
        ),
        Index(
            "uk_xdai_labels_tx_hash_log_idx_evt_raw",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='event'"),
        ),
    )


class ZkSyncEraLabel(EvmBasedLabel):  # type: ignore
    __tablename__ = "zksync_era_labels"

    __table_args__ = (
        Index(
            "ix_zksync_era_labels_addr_block_num",
            "address",
            "block_number",
            unique=False,
        ),
        Index(
            "ix_zksync_era_labels_addr_block_ts",
            "address",
            "block_timestamp",
            unique=False,
        ),
        Index(
            "uk_zksync_era_labels_tx_hash_tx_call",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer' and label_type='tx_call'"),
        ),
        Index(
            "uk_zksync_era_labels_tx_hash_log_idx_evt",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer' and label_type='event'"),
        ),
        Index(
            "uk_zksync_era_labels_tx_hash_tx_call_raw",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='tx_call'"),
        ),
        Index(
            "uk_zksync_era_labels_tx_hash_log_idx_evt_raw",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='event'"),
        ),
    )


class ZkSyncEraSepoliaLabel(EvmBasedLabel):  # type: ignore
    __tablename__ = "zksync_era_sepolia_labels"

    __table_args__ = (
        Index(
            "ix_zksync_era_sepolia_labels_addr_block_num",
            "address",
            "block_number",
            unique=False,
        ),
        Index(
            "ix_zksync_era_sepolia_labels_addr_block_ts",
            "address",
            "block_timestamp",
            unique=False,
        ),
        Index(
            "uk_zksync_era_sepolia_labels_tx_hash_tx_call",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer' and label_type='tx_call'"),
        ),
        Index(
            "uk_zksync_era_sepolia_labels_tx_hash_log_idx_evt",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer' and label_type='event'"),
        ),
        Index(
            "uk_zksync_era_sepolia_labels_tx_hash_tx_call_raw",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='tx_call'"),
        ),
        Index(
            "uk_zksync_era_sepolia_labels_tx_hash_log_idx_evt_raw",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='event'"),
        ),
    )


class BaseLabel(EvmBasedLabel):  # type: ignore
    __tablename__ = "base_labels"

    __table_args__ = (
        Index(
            "ix_base_labels_addr_block_num",
            "address",
            "block_number",
            unique=False,
        ),
        Index(
            "ix_base_labels_addr_block_ts",
            "address",
            "block_timestamp",
            unique=False,
        ),
        Index(
            "uk_base_labels_tx_hash_tx_call",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer' and label_type='tx_call'"),
        ),
        Index(
            "uk_base_labels_tx_hash_log_idx_evt",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer' and label_type='event'"),
        ),
        Index(
            "uk_base_labels_tx_hash_tx_call_raw",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='tx_call'"),
        ),
        Index(
            "uk_base_labels_tx_hash_log_idx_evt_raw",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='event'"),
        ),
    )


class ArbitrumNovaLabel(EvmBasedLabel):  # type: ignore
    __tablename__ = "arbitrum_nova_labels"

    __table_args__ = (
        Index(
            "ix_arbitrum_nova_labels_addr_block_num",
            "address",
            "block_number",
            unique=False,
        ),
        Index(
            "ix_arbitrum_nova_labels_addr_block_ts",
            "address",
            "block_timestamp",
            unique=False,
        ),
        Index(
            "uk_arbitrum_nova_labels_tx_hash_tx_call",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer' and label_type='tx_call'"),
        ),
        Index(
            "uk_arbitrum_nova_labels_tx_hash_log_idx_evt",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer' and label_type='event'"),
        ),
        Index(
            "uk_arbitrum_nova_labels_tx_hash_tx_call_raw",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='tx_call'"),
        ),
        Index(
            "uk_arbitrum_nova_labels_tx_hash_log_idx_evt_raw",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='event'"),
        ),
    )


class ArbitrumOneLabel(EvmBasedLabel):  # type: ignore
    __tablename__ = "arbitrum_one_labels"

    __table_args__ = (
        Index(
            "ix_arbitrum_one_labels_addr_block_num",
            "address",
            "block_number",
            unique=False,
        ),
        Index(
            "ix_arbitrum_one_labels_addr_block_ts",
            "address",
            "block_timestamp",
            unique=False,
        ),
        Index(
            "uk_arbitrum_one_labels_tx_hash_tx_call",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer' and label_type='tx_call'"),
        ),
        Index(
            "uk_arbitrum_one_labels_tx_hash_log_idx_evt",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer' and label_type='event'"),
        ),
        Index(
            "uk_arbitrum_one_labels_tx_hash_tx_call_raw",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='tx_call'"),
        ),
        Index(
            "uk_arbitrum_one_labels_tx_hash_log_idx_evt_raw",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='event'"),
        ),
    )


class ArbitrumSepoliaLabel(EvmBasedLabel):  # type: ignore
    __tablename__ = "arbitrum_sepolia_labels"

    __table_args__ = (
        Index(
            "ix_arbitrum_sepolia_labels_addr_block_num",
            "address",
            "block_number",
            unique=False,
        ),
        Index(
            "ix_arbitrum_sepolia_labels_addr_block_ts",
            "address",
            "block_timestamp",
            unique=False,
        ),
        Index(
            "uk_arbitrum_sepolia_labels_tx_hash_tx_call",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer' and label_type='tx_call'"),
        ),
        Index(
            "uk_arbitrum_sepolia_labels_tx_hash_log_idx_evt",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer' and label_type='event'"),
        ),
        Index(
            "uk_arbitrum_sepolia_labels_tx_hash_tx_call_raw",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='tx_call'"),
        ),
        Index(
            "uk_arbitrum_sepolia_labels_tx_hash_log_idx_evt_raw",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='event'"),
        ),
    )


class Game7OrbitArbitrumSepoliaLabel(EvmBasedLabel):  # type: ignore
    __tablename__ = "game7_orbit_arbitrum_sepolia_labels"

    __table_args__ = (
        Index(
            "ix_g7o_arbitrum_sepolia_labels_addr_block_num",
            "address",
            "block_number",
            unique=False,
        ),
        Index(
            "ix_g7o_arbitrum_sepolia_labels_addr_block_ts",
            "address",
            "block_timestamp",
            unique=False,
        ),
        Index(
            "uk_g7o_arbitrum_sepolia_labels_tx_hash_tx_call",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer' and label_type='tx_call'"),
        ),
        Index(
            "uk_g7o_arbitrum_sepolia_labels_tx_hash_log_idx_evt",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer' and label_type='event'"),
        ),
        Index(
            "uk_g7o_arbitrum_sepolia_labels_tx_hash_tx_call_raw",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='tx_call'"),
        ),
        Index(
            "uk_g7o_arbitrum_sepolia_labels_tx_hash_log_idx_evt_raw",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='event'"),
        ),
    )


class XaiLabel(EvmBasedLabel):  # type: ignore
    __tablename__ = "xai_labels"

    __table_args__ = (
        Index(
            "ix_xai_labels_addr_block_num",
            "address",
            "block_number",
            unique=False,
        ),
        Index(
            "ix_xai_labels_addr_block_ts",
            "address",
            "block_timestamp",
            unique=False,
        ),
        Index(
            "uk_xai_labels_tx_hash_tx_call",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer' and label_type='tx_call'"),
        ),
        Index(
            "uk_xai_labels_tx_hash_log_idx_evt",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer' and label_type='event'"),
        ),
        Index(
            "uk_xai_labels_tx_hash_tx_call_raw",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='tx_call'"),
        ),
        Index(
            "uk_xai_labels_tx_hash_log_idx_evt_raw",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='event'"),
        ),
    )


class XaiSepoliaLabel(EvmBasedLabel):  # type: ignore
    __tablename__ = "xai_sepolia_labels"

    __table_args__ = (
        Index(
            "ix_xai_sepolia_labels_addr_block_num",
            "address",
            "block_number",
            unique=False,
        ),
        Index(
            "ix_xai_sepolia_labels_addr_block_ts",
            "address",
            "block_timestamp",
            unique=False,
        ),
        Index(
            "uk_xai_sepolia_labels_tx_hash_tx_call",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer' and label_type='tx_call'"),
        ),
        Index(
            "uk_xai_sepolia_labels_tx_hash_log_idx_evt",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer' and label_type='event'"),
        ),
        Index(
            "uk_xai_sepolia_labels_tx_hash_tx_call_raw",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='tx_call'"),
        ),
        Index(
            "uk_xai_sepolia_labels_tx_hash_log_idx_evt_raw",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='event'"),
        ),
    )


class AvalancheLabel(EvmBasedLabel):  # type: ignore
    __tablename__ = "avalanche_labels"

    __table_args__ = (
        Index(
            "ix_avalanche_labels_addr_block_num",
            "address",
            "block_number",
            unique=False,
        ),
        Index(
            "ix_avalanche_labels_addr_block_ts",
            "address",
            "block_timestamp",
            unique=False,
        ),
        Index(
            "uk_avalanche_labels_tx_hash_tx_call",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer' and label_type='tx_call'"),
        ),
        Index(
            "uk_avalanche_labels_tx_hash_log_idx_evt",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer' and label_type='event'"),
        ),
        Index(
            "uk_avalanche_labels_tx_hash_tx_call_raw",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='tx_call'"),
        ),
        Index(
            "uk_avalanche_labels_tx_hash_log_idx_evt_raw",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='event'"),
        ),
    )


class AvalancheFujiLabel(EvmBasedLabel):  # type: ignore
    __tablename__ = "avalanche_fuji_labels"

    __table_args__ = (
        Index(
            "ix_avalanche_fuji_labels_addr_block_num",
            "address",
            "block_number",
            unique=False,
        ),
        Index(
            "ix_avalanche_fuji_labels_addr_block_ts",
            "address",
            "block_timestamp",
            unique=False,
        ),
        Index(
            "uk_avalanche_fuji_labels_tx_hash_tx_call",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer' and label_type='tx_call'"),
        ),
        Index(
            "uk_avalanche_fuji_labels_tx_hash_log_idx_evt",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer' and label_type='event'"),
        ),
        Index(
            "uk_avalanche_fuji_labels_tx_hash_tx_call_raw",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='tx_call'"),
        ),
        Index(
            "uk_avalanche_fuji_labels_tx_hash_log_idx_evt_raw",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='event'"),
        ),
    )


class BlastLabel(EvmBasedLabel):  # type: ignore
    __tablename__ = "blast_labels"

    __table_args__ = (
        Index(
            "ix_blast_labels_addr_block_num",
            "address",
            "block_number",
            unique=False,
        ),
        Index(
            "ix_blast_labels_addr_block_ts",
            "address",
            "block_timestamp",
            unique=False,
        ),
        Index(
            "uk_blast_labels_tx_hash_tx_call",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer' and label_type='tx_call'"),
        ),
        Index(
            "uk_blast_labels_tx_hash_log_idx_evt",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer' and label_type='event'"),
        ),
        Index(
            "uk_blast_labels_tx_hash_tx_call_raw",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='tx_call'"),
        ),
        Index(
            "uk_blast_labels_tx_hash_log_idx_evt_raw",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='event'"),
        ),
    )


class BlastSepoliaLabel(EvmBasedLabel):  # type: ignore
    __tablename__ = "blast_sepolia_labels"

    __table_args__ = (
        Index(
            "ix_blast_sepolia_labels_addr_block_num",
            "address",
            "block_number",
            unique=False,
        ),
        Index(
            "ix_blast_sepolia_labels_addr_block_ts",
            "address",
            "block_timestamp",
            unique=False,
        ),
        Index(
            "uk_blast_sepolia_labels_tx_hash_tx_call",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer' and label_type='tx_call'"),
        ),
        Index(
            "uk_blast_sepolia_labels_tx_hash_log_idx_evt",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer' and label_type='event'"),
        ),
        Index(
            "uk_blast_sepolia_labels_tx_hash_tx_call_raw",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='tx_call'"),
        ),
        Index(
            "uk_blast_sepolia_labels_tx_hash_log_idx_evt_raw",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='event'"),
        ),
    )


class ProofOfPlayApexLabel(EvmBasedLabel):  # type: ignore
    __tablename__ = "proofofplay_apex_labels"

    __table_args__ = (
        Index(
            "ix_proofofplay_apex_labels_addr_block_num",
            "address",
            "block_number",
            unique=False,
        ),
        Index(
            "ix_proofofplay_apex_labels_addr_block_ts",
            "address",
            "block_timestamp",
            unique=False,
        ),
        Index(
            "uk_proofofplay_apex_labels_tx_hash_tx_call",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer' and label_type='tx_call'"),
        ),
        Index(
            "uk_proofofplay_apex_labels_tx_hash_log_idx_evt",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer' and label_type='event'"),
        ),
        Index(
            "uk_proofofplay_apex_labels_tx_hash_tx_call_raw",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='tx_call'"),
        ),
        Index(
            "uk_proofofplay_apex_labels_tx_hash_log_idx_evt_raw",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='event'"),
        ),
    )


class StarknetLabel(EvmBasedLabel):  # type: ignore
    __tablename__ = "starknet_labels"

    __table_args__ = (
        Index(
            "ix_starknet_labels_addr_block_num",
            "address",
            "block_number",
            unique=False,
        ),
        Index(
            "ix_starknet_labels_addr_block_ts",
            "address",
            "block_timestamp",
            unique=False,
        ),
        Index(
            "uk_starknet_labels_tx_hash_tx_call",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer' and label_type='tx_call'"),
        ),
        Index(
            "uk_starknet_labels_tx_hash_log_idx_evt",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer' and label_type='event'"),
        ),
        Index(
            "uk_starknet_labels_tx_hash_tx_call_raw",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='tx_call'"),
        ),
        Index(
            "uk_starknet_labels_tx_hash_log_idx_evt_raw",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='event'"),
        ),
    )


class StarknetSepoliaLabel(EvmBasedLabel):  # type: ignore
    __tablename__ = "starknet_sepolia_labels"

    __table_args__ = (
        Index(
            "ix_starknet_sepolia_labels_addr_block_num",
            "address",
            "block_number",
            unique=False,
        ),
        Index(
            "ix_starknet_sepolia_labels_addr_block_ts",
            "address",
            "block_timestamp",
            unique=False,
        ),
        Index(
            "uk_starknet_sepolia_labels_tx_hash_tx_call",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer' and label_type='tx_call'"),
        ),
        Index(
            "uk_starknet_sepolia_labels_tx_hash_log_idx_evt",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer' and label_type='event'"),
        ),
        Index(
            "uk_starknet_sepolia_labels_tx_hash_tx_call_raw",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='tx_call'"),
        ),
        Index(
            "uk_starknet_sepolia_labels_tx_hash_log_idx_evt_raw",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='event'"),
        ),
    )


class MantleLabel(EvmBasedLabel):  # type: ignore
    __tablename__ = "mantle_labels"

    __table_args__ = (
        Index(
            "ix_mantle_labels_addr_block_num",
            "address",
            "block_number",
            unique=False,
        ),
        Index(
            "ix_mantle_labels_addr_block_ts",
            "address",
            "block_timestamp",
            unique=False,
        ),
        Index(
            "uk_mantle_labels_tx_hash_tx_call",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer' and label_type='tx_call'"),
        ),
        Index(
            "uk_mantle_labels_tx_hash_log_idx_evt",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer' and label_type='event'"),
        ),
        Index(
            "uk_mantle_labels_tx_hash_tx_call_raw",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='tx_call'"),
        ),
        Index(
            "uk_mantle_labels_tx_hash_log_idx_evt_raw",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='event'"),
        ),
    )


class MantleSepoliaLabel(EvmBasedLabel):  # type: ignore
    __tablename__ = "mantle_sepolia_labels"

    __table_args__ = (
        Index(
            "ix_mantle_sepolia_labels_addr_block_num",
            "address",
            "block_number",
            unique=False,
        ),
        Index(
            "ix_mantle_sepolia_labels_addr_block_ts",
            "address",
            "block_timestamp",
            unique=False,
        ),
        Index(
            "uk_mantle_sepolia_labels_tx_hash_tx_call",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer' and label_type='tx_call'"),
        ),
        Index(
            "uk_mantle_sepolia_labels_tx_hash_log_idx_evt",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer' and label_type='event'"),
        ),
        Index(
            "uk_mantle_sepolia_labels_tx_hash_tx_call_raw",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='tx_call'"),
        ),
        Index(
            "uk_mantle_sepolia_labels_tx_hash_log_idx_evt_raw",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='event'"),
        ),
    )


class ImxZkevmLabel(EvmBasedLabel):  # type: ignore
    __tablename__ = "imx_zkevm_labels"

    __table_args__ = (
        Index(
            "ix_imx_zkevm_labels_addr_block_num",
            "address",
            "block_number",
            unique=False,
        ),
        Index(
            "ix_imx_zkevm_labels_addr_block_ts",
            "address",
            "block_timestamp",
            unique=False,
        ),
        Index(
            "uk_imx_zkevm_labels_tx_hash_tx_call",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer' and label_type='tx_call'"),
        ),
        Index(
            "uk_imx_zkevm_labels_tx_hash_log_idx_evt",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer' and label_type='event'"),
        ),
        Index(
            "uk_imx_zkevm_labels_tx_hash_tx_call_raw",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='tx_call'"),
        ),
        Index(
            "uk_imx_zkevm_labels_tx_hash_log_idx_evt_raw",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='event'"),
        ),
    )


class ImxZkevmSepoliaLabel(EvmBasedLabel):  # type: ignore
    __tablename__ = "imx_zkevm_sepolia_labels"

    __table_args__ = (
        Index(
            "ix_imx_zkevm_sepolia_labels_addr_block_num",
            "address",
            "block_number",
            unique=False,
        ),
        Index(
            "ix_imx_zkevm_sepolia_labels_addr_block_ts",
            "address",
            "block_timestamp",
            unique=False,
        ),
        Index(
            "uk_imx_zkevm_sepolia_labels_tx_hash_tx_call",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer' and label_type='tx_call'"),
        ),
        Index(
            "uk_imx_zkevm_sepolia_labels_tx_hash_log_idx_evt",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer' and label_type='event'"),
        ),
        Index(
            "uk_imx_zkevm_sepolia_labels_tx_hash_tx_call_raw",
            "transaction_hash",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='tx_call'"),
        ),
        Index(
            "uk_imx_zkevm_sepolia_labels_tx_hash_log_idx_evt_raw",
            "transaction_hash",
            "log_index",
            unique=True,
            postgresql_where=text("label='seer-raw' and label_type='event'"),
        ),
    )
